# coding=utf-8
"""
@Project : kylin-adaptation-tools
@Time    : 2023/11/16 17:04
@Author  : wangbin wangchunli
"""
from PySide2.QtWidgets import QDialog
from PySide2.QtCore import Signal
from src.view.authorization import Ui_authorization
from src.utils.utils import check_passwd


class AuthWindow(QDialog, Ui_authorization):
    """
    授权窗口的controller，实现授权窗体业务逻辑
    """
    signal_done = Signal(str)
    signal_cancel = Signal(str)
    signal_passwd = Signal(str)

    def __init__(self):
        super(AuthWindow, self).__init__()
        self.setupUi(self)
        self.pushButton_title_bar_close.clicked.connect(self.close_dialog)
        self.pushButton_cancel.clicked.connect(self.cancel_dialog)
        self.pushButton_auth.clicked.connect(self.commit)
        # 将密码输入框的enter事件和提交按钮绑定
        self.lineEdit_password.returnPressed.connect(self.commit)
        self.auth_label_passwd_error.setVisible(False)

    def cancel_dialog(self):
        self.signal_cancel.emit('cancel')
        self.close_dialog()

    def close_dialog(self):  # 子窗体自定义事件
        self.close()

    def commit(self):
        passwd = self.lineEdit_password.text() or ''
        if not passwd:
            return
        res = check_passwd(passwd)
        if not res:
            self.auth_label_passwd_error.setVisible(True)
            return
        else:
            self.auth_label_passwd_error.setVisible(False)
        # print(f'  授权窗口 {user}, {passwd},{self.signal_done} 账户或者密码。。。。')
        self.signal_passwd.emit(passwd)
        self.signal_done.emit('done')
        self.close_dialog()
