# coding=utf-8
"""
@Project : kylin-adaptation-tools
@Time    : 2023/8/16 15:10
@Author  : wangbin
"""
import os.path
import shutil
import time
from PySide2.QtCore import QThread, Signal
from src.setting import REPORT_PATH
from src.utils.operations import Operation
# 取消生成doc文件，注释对docutils的引用
# from src.utils.doc_utils import DocUtils
from src.utils.csv_utils import CSV
from src.utils.logger import logger


class RunTestThread(QThread):
    """
    qt 子线程，用于执行测试脚本
    """
    signal_running = Signal(str)
    signal_debcheck_done = Signal(str)
    signal_done = Signal(str)
    signal_report_html = Signal(str)
    # signal_report_doc = Signal(str)
    signal_report_csv = Signal(str)

    def __init__(self):
        super().__init__()
        logger.info('测试执行线程 init 。。。 。。。')
        self.package = None
        self.pkg_type = 'software'
        self.auth_passwd = ''

    def set_package(self, package):
        self.package = package

    def set_type(self, type_index):
        types = {
            0: 'software',
            1: 'peripheral'
        }
        self.pkg_type = types.get(type_index)
        logger.info(f'测试执行线程 包类型为：{self.pkg_type} 。。。 。。。')

    def set_passwd(self, auth_passwd):
        self.auth_passwd = auth_passwd

    def run(self):
        """
        执行测试，并生成报告文件
        :return:
        """
        logger.info(f'测试执行线程 执行开始 。。。 。。。')
        self.signal_running.emit('running')

        debcheck_report_detail, debcheck_report_summary, report_pkg_info = Operation().run_debcheck(self.package, self.pkg_type)
        # self.signal_report_csv.emit(debcheck_report)
        self.signal_debcheck_done.emit('done')
        self.signal_done.emit('10')

        report_env, report_case = Operation().run_blazers(self.package, self.auth_passwd)
        # self.signal_report_html.emit(report_html)

        # env_info = CSV(report_env).csv_reader_transverse()
        # case_result = CSV(report_case).csv_reader()
        # docx = DocUtils(self.pkg_type)
        # report_doc = docx.output_document(env_info, case_result, REPORT_PATH)
        # self.signal_report_doc.emit(debcheck_report)

        report_html = Operation().run_report_tool(report_env, debcheck_report_detail, report_case, report_pkg_info)

        if os.path.exists(report_html):
            report_name = report_html.split('/')[-1]
            tool_path = os.getcwd()
            shutil.copy2(report_html, f'{tool_path}/Reports/{report_name}')
            report_html = f'{tool_path}/Reports/{report_name}'

        self.signal_report_html.emit(report_html)

        self.signal_done.emit('100')
        self.signal_running.emit('free')


class ProcessPercentThread(QThread):
    """
    qt 子线程，进度条百分比
    """
    signal_percent = Signal(str)
    signal_status = Signal(str)

    def __init__(self):
        super().__init__()
        logger.info('进度条百分比线程 init 。。。 。。。')
        self.percent = None
        self.status = None

    def stop(self):
        self.status = 'stop'

    def run(self):
        """
        更新当前百分比
        :return:
        """
        logger.info(f'进度条百分比线程 执行开始 。。。 。。。')
        # for i in range(10, 100):
        #     if self.status == 'stop':
        #         self.signal_percent.emit('100')
        #         logger.info('进度条百分比线程 结束 。。。 。。。')
        #         return
        #
        #     self.signal_percent.emit(str(i))
        #     time.sleep(2)
        time.sleep(3)
        loop = True
        while loop:
            time.sleep(2)
            if self.status == 'stop':
                self.signal_percent.emit('100')
                logger.info('进度条百分比线程 结束 。。。 。。。')
                return

            percent = Operation().read_percent()
            percent = max(percent, 10)
            self.signal_percent.emit(str(percent))


