# coding=utf-8
"""
@Project : kylin-adaptation-tools
@Time    : 2023/8/16 15:06
@Author  : wangbin
"""
import os.path
import subprocess
from PySide2.QtWidgets import QWidget, QFileDialog
from src.view.window import Ui_MainWindow
from PySide2 import QtWidgets
from PySide2.QtCore import QTimer
from src.controller.about import AboutWindow
from src.controller.authorization import AuthWindow
from src.controller.child_thread import RunTestThread, ProcessPercentThread
from src.utils.logger import logger


class MainWindow(QtWidgets.QMainWindow):
    """
    主窗体的controller，实现主窗体业务逻辑
    """

    def __init__(self):
        super().__init__()
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)
        self.set_step('prepare')
        self.init_action()
        self.test_suite = dict()
        self.process_value = 0
        self.package = None
        self.auth_passwd = None
        self.auth_done = None
        self.running = False
        logger.info('主窗体初始化完成 。。。 。。。')

    def init_action(self):
        """
        控件-动作 绑定到 方法
        """
        self.ui.pushButton_test_start.clicked.connect(self.prepare_test)
        self.ui.pushButton_browser_file.clicked.connect(self.openfile)
        self.ui.pushButton_title_bar_min.clicked.connect(self.set_min_window)
        self.ui.pushButton_title_bar_close.clicked.connect(self.close_window)
        self.ui.pushButton_title_bar_menu.clicked.connect(self.show_about)
        self.ui.pushButton_report_html.clicked.connect(self.show_report_html)
        self.ui.label_test_report_html.clicked.connect(self.show_report_html)
        self.ui.pushButton_return.clicked.connect(self.reset_test)

    def show_about(self):
        """
        显示授权窗口
        :return:
        """
        self.about_window = AboutWindow()
        self.about_window.show()
        logger.info('打开关于窗口 。。。 。。。')

    def set_min_window(self):
        """
        窗体最小化
        :return:
        """
        self.showMinimized()

    def close_window(self):
        """
        关闭窗口，退出程序
        :return:
        """
        logger.info('关闭程序 。。。 。。。')
        self.close()

    def _mousePressEvent(self, event):
        """
        自定义标题栏，鼠标点击事件处理
        :param event:
        :return:
        """
        self.isPressed = True
        self.startPos = event.globalPos()
        return QWidget().mousePressEvent(event)

    def _mouseReleaseEvent(self, event):
        """
        自定义标题栏，鼠标释放事件处理
        :param event:
        :return:
        """
        self.isPressed = False
        return QWidget().mouseReleaseEvent(event)

    def _mouseMoveEvent(self, event):
        """
        自定义标题栏，鼠标拖动事件处理
        :param event:
        :return:
        """
        if self.isPressed:
            if self.win.isMaximized:
                self.win.showNormal()

            movePos = event.globalPos() - self.startPos
            self.startPos = event.globalPos()
            self.win.move(self.win.pos() + movePos)

        return QWidget().mouseMoveEvent(event)

    def prepare_test(self):
        """
        准备执行测试，先授权状态检查
        :return:
        """
        self.set_step('prepare')
        logger.info(f'准备测试阶段，auth_done状态为{self.auth_done} 。。。 。。。')
        if not self.package:
            return
        if self.auth_done:
            logger.info(f'已授权， 直接测试 。。。 。。。')
            self.run_test()
        else:
            logger.info(f'未授权， 先行授权 。。。 。。。')
            self.get_auth()

    def get_auth(self):
        """
        调用授权窗口
        :return:
        """
        self.set_step('auth')
        self.auth_window = AuthWindow()
        self.auth_window.signal_done[str].connect(self.set_auth_done)
        self.auth_window.signal_passwd[str].connect(self.set_passwd)
        self.auth_window.signal_done[str].connect(self.run_test)
        self.auth_window.signal_cancel[str].connect(self.set_auth_cancel)
        # print(f'  授权窗口 {self.auth_user}, {self.auth_passwd},{self.auth_done} 账户或者密码。。。。')
        self.auth_window.show()

    def run_test(self):
        """
        执行测试业务
        :return:
        """
        logger.info(f'测试开始执行 。。。 。。。')
        if self.running:
            return
        self.set_step('running')
        type_index = self.ui.comboBox_select_type.currentIndex()
        self.echo_process('0')

        # 获取兼容测试进度的线程
        self.pthread = ProcessPercentThread()
        self.pthread.signal_percent[str].connect(self.echo_process)

        # 启用一个子线程进行测试动作
        self.nthread = RunTestThread()
        if not self.nthread.isRunning():
            self.nthread.set_package(self.package)
            self.nthread.set_type(type_index)
            self.nthread.set_passwd(self.auth_passwd)
            self.nthread.start()
        # self.nthread.signal_done[str].connect(self.echo_info)
        self.nthread.signal_done[str].connect(self.echo_process)
        self.nthread.signal_running[str].connect(self.set_running_status)
        self.nthread.signal_report_html[str].connect(self.echo_report_html)
        # 包规范检测工具执行完后启动blazer的进度监控
        self.nthread.signal_debcheck_done[str].connect(self.pthread.start)
        self.nthread.start()

    def set_passwd(self, passwd):
        self.auth_passwd = passwd

    def set_auth_done(self):
        self.auth_done = 'done'

    def set_auth_cancel(self):
        """
        授权窗口 取消按钮业务
        :return:
        """
        self.set_step('prepare')

    def set_running_status(self, string):
        if string == 'running':
            self.running = True
        elif string == 'finish':
            self.running = False
        else:
            self.running = False

    def echo_report_html(self, string):
        self.report_html = string

    def echo_report_csv(self, string):
        self.report_csv = string

    def echo_process(self, string):
        """
        测试进度条根据业务进行数值变化
        :param string:
        :return:
        """
        new_value = int(float(string))
        self.process_value = new_value
        self.ui.progressBar_test.setValue(self.process_value)
        if new_value == 100:
            self.pthread.stop()
            QTimer.singleShot(1000, self.set_step)
        else:
            pass
            # self.ui.textBrowser_output.append(f'{self.process_value}进行中，加油加油')

    def set_step(self, step='finish'):
        """
        组件根据用户使用步骤进行显隐切换
        :param step:
        :return:
        """
        body_index = {
            'prepare': 0,
            'auth': 0,
            'running': 1,
            'finish': 2,
        }
        self.ui.tabWidget_body.setCurrentIndex(body_index.get(step))
        logger.info(body_index.get(step))
        logger.info(f" 当前阶段是 {step}")
        # print(f" 测试执行按钮状态是 {steps.get(step).get('button_test_start')}")

    def openfile(self):
        """
        获取本地文件方法
        :return:
        """
        # openfile_name = QFileDialog.getOpenFileName(self, '选择待测包', '', 'all file(*)')
        # 文件管理器样式不再继承父类，使用系统默认样式
        openfile_name = QFileDialog.getOpenFileName(None, '选择待测包', '', 'deb package(*.deb)')
        # print(openfile_name)
        logger.info(f'测试包是 {openfile_name[0]} 。。。 。。。')
        if openfile_name[0]:
            self.package = openfile_name[0]
            self.ui.lineEdit_filepath.setText(openfile_name[0])

    def show_report_csv(self):
        """
        以默认方式打开csv报告
        :return:
        """
        # os.startfile(self.report_html) # on Windows PC
        subprocess.call(["xdg-open", self.report_csv])

    def show_report_html(self):
        """
        以默认方式打开html报告
        :return:
        """
        # os.startfile(self.report_html) # on Windows PC
        subprocess.call(["xdg-open", self.report_html])

    def reset_test(self):
        if self.running:
            return
        self.ui.lineEdit_filepath.setText('')
        self.set_step('prepare')
