# 自动化适配框架 Blazers

## 工具简介

        选取了先天兼容性好的编程语言Shell，能够在多个架构（X86、ARM、MIPS等）下一键执行，省去了框架部署及安装依赖的时间，并且无需解决平台适配问题；
        框架采用了分层设计思想，包括：用例组织层、执行调度层、基础组件层（公共库、调度组件、日志组件、报告组件、邮件组件）、数据收集层；
        测试人员只需要关注测试脚本的开发即可；

## 运行方式

### 配置要求

        本框架适用于主流的arm、x86、mips等平台，适用于银河麒麟V10及以后的系统

### 安装环境（安装依赖）

| 组件   | 组件描述         | 可获得性                   |
|------ | --------------- | ------------------------- |
| curl  | 7.68.0 及以上版本 | 可使用 apt install 命令安装 |
| mutt  | 1.13.2 及以上版本 | 可使用 apt install 命令安装 |
| msmtp | 1.8.6 及以上版本  | 可使用 apt install 命令安装 |

### 启动命令

    启动命令为执行run.sh，详细使用方法如下：

    1. 筛选用例参数，只能同时使用一个
    	(1) 不加参数运行，默认执行所有用例  
		(2) --run caselist				通过用例名运行指定用例  
		(3) --include tag				运行包含tag的用例运行  
		(4) --include tag1ANDtag2		运行同时包含tag1和tag2的用例  
		(5) --include tag1NOTtag2		运行包含tag1但不包含tag2的用例  
		(6) --exclude tag				运行不包含tag的用例
    2. 配置参数
        -p, --passwd password           配置测试机密码，必填
    3. 其他参数
        按照具体工具的定义填写

    起同样作用的参数如筛选用例参数最多使用一个
    起不同作用的参数如筛选用例参数和配置测试机密码参数可以搭配使用

* 使用示例
    * 执行所有用例  
        `./run.sh -p 123`
    * 指定运行用例 testcase01  
        `./run.sh -p 123 --run testcase01`
    * 运行包含标签 p0 的用例  
        `./run.sh -p 123 --include p0`
    * 运行包含标签 p0 和 smoke 的用例  
        `./run.sh -p 123 --include p0ANDsmoke`
    * 运行包含标签 p0 但不包含 smoke 的用例  
        `./run.sh -p 123 --include p0NOTsmoke` 
    * 运行不包含标签 p1 的用例  
        `./run.sh -p 123 --exclude p1` 
    * 如果密码中含有特殊字符如!@#等，在终端输入时需要转义  
        `./run.sh -p qwer1234\!\@\#\$`
    * 如果测试执行过程中断，需要继续执行  
        `./run.sh`

## 使用方法

### 框架使用方法

1. 下载代码 [Blazers](https://gitlab.kylin.com/qa/blazers)
2. 解压缩后进入工程目录
3. 在框架目录下运行run.sh文件

### 测试用例配置： 

1. 进入框架目录
2. 新建项目文件夹，要求使用项目名称，如TestDemo
3. 进入项目文件夹，新建测试套件文件夹，命名为casepool。
4. 进入套件文件夹，新建测试用例文件，要求是shell文件，要求名称包含testcase字段。
5. 测试用例文件开头必须包含Tags=”[]”变量，用户可在括号内配置标签，多个标签使用逗号分割，用户在运行框架时可根据定义的标签筛选用例。
6. 修改config中的项目配置文件configuration.ini，将PROJECT_NAME字段配置为项目文件夹名。

### 其他配置

1. 初始化配置
    如果项目中用到一些初始化的操作，可作为独立的脚本，在项目文件夹下新建setup文件夹，将初始化的脚本放置于此，整个项目的初始化脚本命名为project_setup.sh，针对每个应用的初始化脚本命名为app_setup.sh。
2. 测试参数配置  
       支持指定整套用例和每条用例的执行次数，可按照参数列表执行或者指定次数，请在control/control.ini文件中配置。
3. 邮件发送配置项 a.是否发送邮件 b.发件人邮箱 c.发件人邮箱密码 d.收件人邮箱，多人可用半角逗号隔开 
4. 记录的日志等级 日志等级由低到高为ALL < DEBUG < INFO < WARN < ERROR < CRITICAL，从低到高对应数字1-6，填写对应优先级数字 
5. 分批测试配置  
   批量测试支持分批次进行，用户可自定义每批次的测试数量，每批次测试完成后工具自动还原初始测试环境进行下一批次的测试，保证测试稳定性。  
   需要修改config中的项目配置文件configuration.ini
   1. 不需要定量还原测试环境时置BATCH为0（默认为此）
   2. 需要定量还原测试环境时将BATCH置为1，同时设置BATCH_NUMBER为需要的量
   3. 注意事项：测试前需要保证机器能够在重启之后自动联网，主要涉及服务器系统，桌面未发现此类问题，步骤如下  
      1. 查看需要连接的网络名称，如ens192
      2. 编辑文件 /etc/sysconfig/network-scripts/ifcfg-ens192，将其中的ONBOOT选项值改为yes，保存文件退出即可

## 目录结构

    ├── config                  ——————配置文件夹
    │   ├── configuration.ini       ——————配置文件
    │   └── requirements.txt        ——————运行依赖
    ├── control                 ——————控制文件夹
    │   ├── control.ini             ——————用例运行控制配置文件
    │   └── controllib.sh           ——————用例运行控制相关方法库
    ├── log                     ——————存放日志
    │   └── caselog               ——————存放每个应用的用例日志
    ├── pack                    ——————存放打包的结果
    ├── pub                     ——————公共方法库
    │   ├── baselib.sh              ——————基础方法库
    │   ├── caseschedule.sh         ——————用例调度方法库
    │   ├── datastore.sh            ——————数据存储方法库
    │   ├── envmaintain.sh          ——————系统备份恢复方法库
    │   ├── log.sh                  ——————日志方法库
    │   ├── newsnotify.sh           ——————结果通知方法库
    │   └── report.sh               ——————报告生成方法库
    ├── README.md               ——————工具说明文件
    ├── report                  ——————存放报告
    │   └── subreport               ——————存放每个应用的报告
    ├── run.sh                  ——————工具运行入口
    ├── template                ——————存放测试报告模板 
    └── tools                   ——————配合使用的其他工具
        └── sysrestore              ——————备份还原工具

## Blazers 框架中Shell公共方法

* **log_debug**
    * 输出DEBUG级别日志
    * 使用方法：  
    `log_debug "这是一条debug日志"`

* **log_info**
    * 输出INFO级别日志
    * 使用方法：  
	`log_info "这是一条info日志"`

* **log_warn**
    * 输出WARN级别日志
    * 使用方法：  
    `log_warn "这是一条warn日志"`

* **log_error**
    * 输出ERROR级别日志
    * 使用方法：  
    `log_error "这是一条error日志"`

* **log_critical**
    * 输出critical级别日志
    * 使用方法：  
    `log_critical "这是一条critical日志"`

* **assert_equal_string**
    * 判断两个字符串是否相等
    * 使用方法：  
    `assert_equal_string "str1" "str2"`

* **assert_not_equal_string**
    * 判断两个字符串是否不相等
    * 使用方法：  
    `assert_not_equal_string "str1" "str2"`

* **assert_equal_num**
    * 判断两个数字是否相等
    * 使用方法：  
    `assert_equal_num 1 2`

* **assert_not_equal_num**
    * 判断两个数字是否不相等
    * 使用方法：  
    `assert_not_equal_num 1 2`

* **assert_string_contain_by_file**
    * 判断文件中是否包含某个关键字
    * 使用方法：  
    `assert_string_contain_by_file "str" "./file"`

* **assert_command_success**
    * 判断命令是否成功
    * 使用方法：  
    `command`  
    `assert_command_success`

* **check_install**
    * 判断软件是否已安装
    * 使用方法：  
    `check_install "weixin"`

* **uninstall_deb**
    * apt卸载应用
    * 使用方法
    `uninstall_deb "weixin"`

* **install_deb**
    * apt安装应用
    * 使用方法：
    `install_deb "weixin"`

## 负责人

    王春力

## 版本更新

    V1.0
    V2.0 
        1. 支持自定义测试报告模板，全新测试报告样式 
        2. 支持在服务器上运行能力，扩展使用场景
        3. 添加了参数模块，支持传入项目自定义参数
    V3.0
        1. 测试报告中添加错误日志显示，方便快速定位问题
        2. 添加安全中心配置检查
    V4.0
        1. 出现异常中断后，支持从中断测试集继续测试
        2. 当有大批量应用需要测试时，支持定量恢复环境以增加测试稳定性