# 兼容性适配认证工具

## 工具简介

	旨在检查麒麟桌面/服务器应用程序的兼容性情况，通过整合软硬件适配基准测试标准，通过自动化与人工复核的方式，针对包规范检查、兼容性检查、运行基础检查（安装、启动、卸载等）、性能、可靠性、安全性等
	多维度定位软硬件应用在不同架构、不同系统版本上的适配问题，并提供详细的适配评估报告。

### 测试用例
#### desktop
| 序号  | 用例编号                    | 用例名称                   |
|-----|-------------------------|------------------------|
| 1   | common-base-01          | 软件包文件检查                |
| 2   | common-base-02          | 包名规范一致性检查              |
| 3   | common-compatibility-01 | 软件依赖库（so文件）查询测试        |
| 4   | common-compatibility-02 | 软件依赖字段（Depends）兼容性检查测试 |
| 5   | common-compatibility-03 | 软件gcc版本兼容性测试           |
| 6   | common-compatibility-04 | 软件glibc版本兼容性测试         |
| 7   | common-compatibility-05 | 内核模块兼容性检查              |
| 8   | common-function-01      | 应用成功更新                 |
| 9   | common-function-02      | 应用成功安装                 |
| 10  | common-function-04      | 应用成功启动                 |
| 11  | common-function-05      | 应用卸载后重装正常              |
| 12  | common-performance-01   | 软件在麒麟桌面操作系统上打开的时长正常    |
| 13  | common-reliability-01   | 软件在麒麟桌面操作系统中重复启动关闭测试   |
| 14  | common-reliability-03   | 软件异常关闭后，软件能正常打开        |
| 15  | common-uninstall-01      | 应用成功卸载                 |
#### server
| 序号  | 用例编号               | 用例名称   |
|-----|--------------------|--------|
| 1   | common-base-01 | 软件包文件检查 |
| 2   | common-base-02 | 包名规范一致性检查 |
| 3   | common-compatibility-01 | 软件依赖库（so文件）查询测试 |
| 4   | common-compatibility-03 | 软件gcc版本兼容性测试 |
| 5   | common-compatibility-04 | 软件glibc版本兼容性测试 |
| 6   | common-function-01 | 应用成功更新 |
| 7   | common-function-02 | 应用成功安装 |
| 8   | common-function-04 | 应用服务启停测试 |
| 9   | common-function-05 | 应用命令调用测试 |
| 10   | common-function-06 | 应用卸载后重装正常 |
| 11   | common-uninstall-01 | 应用成功卸载 |


## 运行方式

本工具需要与代码框架 [Blazers](https://gitlab.kylin.com/qa/blazers/) 结合使用

### 配置要求

    1. 本工具需要运行于银河麒麟桌面/服务器操作系统V10及以后的版本;
    2. 本工具的配置文件中的参数无需用户手动更改。


### 安装环境（安装依赖）
	
| 组件      | 组件描述               | 可获得性                 |
|---------|--------------------|----------------------|
| xdotool | 3.20160805.1 及以上版本 | 可使用 apt install 直接安装 |

### 启动命令

	请结合代码框架blazers使用

## 使用方法

	1. 在blazers框架下执行run.sh，可以配合以下参数使用
		筛选用例选项
    	(1) 不加参数运行，默认执行所有用例
		(2) --run caselist						通过用例名运行指定用例
		(3) --include tag						运行包含tag的用例运行
		(4) --include tag1ANDtag2				运行同时包含tag1和tag2的用例
		(5) --include tag1NOTtag2				运行包含tag1但不包含tag2的用例
		(6) --exclude tag						运行不包含tag的用例

		项目配置选项
		(7) -p|--passwd password				配置测试机密码，必填

		测试列表配置选项，有且只能传入以下一种选项
		(8) --pkg package						测试单包，本地包传入完整路径，源内包传入包名
		(9) --dir packages_path					测试包目录，用于测试大量本地包，将测试包都置于某目录下，传入目录即可
		(10) --list_file test_package_list		测试包列表，要求每行放置一个包

    命令模板：./run.sh -p 测试机密码 测试列表配置选项 筛选用例选项
	举例：
		./run.sh -p qwer1234 --pkg wps-office					配置测试机密码为qwer1234，测试包为源里的wps-office，运行所有用例

		./run.sh -p qwer1234 --include p0 --pkg wps-office		配置测试机密码为qwer1234，测试包为源里的wps-office，运行标签带有p0的用例

		./run.sh -p qwer1234\!\@\#\$ --include p1 --dir /home/kylin
		配置测试机密码为qwer1234!@#$，测试包为/home/kylin下的deb包，运行带有p1标签的用例

		./run.sh -p qwer1234\!\@\#\$ --exclude p0 --list_file /home/kylin/application.list
		配置测试机密码为qwer1234!@#$，测试包列表按照/home/kylin/application.list文件执行，运行不带有p0标签的用例  

	2. 多包测试情况下，支持黑名单机制，用户在config/app.blacklist中写明需要去除的包名（需要和测试列表中相同），格式为一行一个，测试时不会测试黑名单中的应用

## 目录结构

	├── casepool					———————存放测试用例脚本
	│   ├── desktop
	│   │   ├── common-base-01.sh
	│   │   ├── common-base-02.sh
	│   │   ├── common-compatibility-01.sh
	│   │   ├── common-compatibility-02.sh
	│   │   ├── common-compatibility-03.sh
	│   │   ├── common-compatibility-04.sh
	│   │   ├── common-compatibility-05.sh
	│   │   ├── common-function-01.sh
	│   │   ├── common-function-02.sh
	│   │   ├── common-function-04.sh
	│   │   ├── common-function-05.sh
	│   │   ├── common-performance-01.sh
	│   │   ├── common-reliability-01.sh
	│   │   └── common-reliability-03.sh
	│   │   ├── common-uninstall-01
	│   └── server
	│       ├── common-base-01.sh
	│       ├── common-base-02.sh
	│       ├── common-compatibility-01.sh
	│       ├── common-compatibility-03.sh
	│       ├── common-compatibility-04.sh
	│       ├── common-function-01.sh
	│       ├── common-function-02.sh
	│       ├── common-function-04.sh
	│       ├── common-function-05.sh
	│       ├── common-function-06.sh
	│       └── common-uninstall-01
	├── config
	│	├── application.list		——————测试软件列表
	│	└── config					——————工具配置文件
	├── lib
	│   ├── common_function.sh		——————公共方法库
	│   ├── ko_process.sh			——————内核模块文件检查相关方法
	│   └── server_process.sh		——————服务器测试用例相关方法
	├── README.md					——————工具说明文档
	├── resource					——————存放测试包
	└── setup						——————存放工具用到的初始化脚本
		└── app_setup.sh			——————针对每个测试应用的初始化脚本
		└── project_setup.sh		——————针对项目的初始化脚本

## 常见问题

	1. 测试完成后，报告生成在/blazers/report下，report.csv和html展示测试总结果；每个应用会在/blazers/report/subreport下生成自有子报告，展示应用每条用例的结果，结果一栏中：NA代表用例不满足执行条件，未执行；WARNING代表检查项存在兼容性问题，需人工排查验证；Unknown代表用例执行过程有未知错误，需人工排查
	2. 测试完成后，日志生成在/blazers/log下，run_log记录了整个测试的关键执行情况，各应用用例执行的细节记录在/blazers/log/caselog/各应用子log文件中。
	3. 用例测试结果中除了 PASS 和 FAIL，还可能显示
		NA：		用例不满足执行条件，无须执行
		UNKNOWN：	用例执行出现未知错误
		WARNING:	兼容性测试用例特有结果，代表此条用例可能存在兼容性问题，需要用户关注

## 负责人

	王春力

## 版本更新

	V1.0
	V2.0
		1. 支持本地单包、本地多包、源内单包、源内多包等多种测试场景，支持单包测试和批量测试  
		2. 支持服务器适配测试，增加服务器应用基础功能测试用例（安装、启动、卸载） 
		3. 桌面应用测试增加兼容性相关用例，包括软件包依赖、相关库文件、gcc/glibc、引用头文件情况检查 
		4. 使用全新报告模板，新增测试套概念，支持快速查看应用具体测试用例结果 
	V3.0
		1. 支持服务器适配测试，增加规范性、兼容性相关用例（包括软件包相关库文件、gcc/glibc情况检查）   