#!/bin/bash
#######################################################################################
#Function: function of debcheck
#Platform:amd64\arm64\x86
#Author: Fenfen Liu,Chunli Wang
#Date: 2022/03/16
#Company: Kylin Software Co.,Ltd.
#Description: 确认control文件中包名、版本号、架构三个字段符合打包规范，包名符合规范，包名中信息与control中对应字段一致
########################################################################################
source ./kylincompatibilitytest/config/config
source ./kylincompatibilitytest/lib/common_function.sh
source ./config/configuration.conf
source ./pub/baselib.sh
source ./pub/log.sh

Tags="[desktop,base,p0,skip]"
Casenumber="common-base-02"
Casename="包名规范一致性检查"
Description="[common-base-02]：确认control文件中包名、版本号、架构三个字段符合打包规范，包名符合规范，包名中信息与control中对应字段一致"
flag=0
fail_num=0
# 检查关键字段是否符合字段规范
function check_keyword()
{
    local keyword=$1
    local rule=$2
    local loginfo=$3
    echo "${keyword}" | grep -P "${rule}"
    if [ "$?" = "1" ]; then
        fail_num=$[fail_num+1]
        log_error ${loginfo} >> "${CASE_LOG_FILE}"
    fi
}

# 测试步骤：分步检查包规范
function testcase_base02_teststep()
{
    # step1-检查Package字段是否符合规范
    log_info "control文件中包名为：${PKG_PACKAGENAME}，匹配规则为：${DEB_PACKAGE_RULE}"
    check_keyword ${PKG_PACKAGENAME} ${DEB_PACKAGE_RULE} "control文件的Package字段不符合规范"
    # step2-检查Version字段是否符合规范
    log_info "control文件中版本号为：${PKG_VERSION}，匹配规则为：${DEB_VERSION_RULE}"
    check_keyword ${PKG_VERSION} ${DEB_VERSION_RULE} "control文件的Version字段不符合规范"
    # step3-检查Architecture字段是否符合规范
    local architecture=$(dpkg -f ${PKG_PATH} "Architecture")
    log_info "control文件中架构为：${architecture}，匹配规则为：${DEB_ARCHITECTURE_RULE}"
    check_keyword ${architecture} ${DEB_ARCHITECTURE_RULE} "control文件的Architecture字段不符合规范"
    # step4-检查deb_name是否符合规范
    log_info "软件包名称为：${PKG_SOFTNAME}，匹配规则为：${DEB_PACK_RULE}"
    check_keyword ${PKG_SOFTNAME} ${DEB_PACK_RULE} "deb包的命名不符合规范"
    # step5-包名称（deb_name）与control文件Package、Version、Architecture字段内容一致
    local pkg_name="${PKG_PACKAGENAME}_${PKG_VERSION}_${architecture}.deb"
    pkg_name=${pkg_name//\+/\\\+}
    pkg_name=${pkg_name//\./\\\.}
    pkg_name="^"${pkg_name}"$"
    log_info "软件名称为：${PKG_SOFTNAME}，匹配规则为：${pkg_name}"
    check_keyword ${PKG_SOFTNAME} ${pkg_name} "deb包命名字段与control文件不一致"

    if [ "${fail_num}" = "0" ]; then
        flag=0
    else
        flag=1
    fi
}

function TestCase_common_base_02()
{
    write_caseinfo_to_log "${Casenumber}" "${Description}"
    testcase_base02_teststep
    write_endinfo_to_log "${Casenumber}"
}

# 执行测试
TestCase_common_base_02 >> "${CASE_LOG_FILE}"
# assert 测试步骤是否全部PASS
assert_result_by_flag "${flag}" "${Casenumber}"
