#!/bin/bash
#######################################################################################
#Function: check the existence of library files required by ELF files
#Platform:all
#Author: RongJia Du
#Date: 2022/05/25
#Company: Kylin Software Co.,Ltd.
#Description: 检查应用ELF文件所需动态库so文件是否有缺失
########################################################################################
source ./kylincompatibilitytest/config/config
source ./kylincompatibilitytest/lib/common_function.sh
source ./config/configuration.conf
source ./pub/baselib.sh
source ./pub/log.sh

Tags="[desktop,compatibility,p0]"
Casenumber="common-compatibility-01"
Casename="软件依赖库（so文件）查询测试"
Description="[common-compatibility-01]：软件在指定架构、操作系统上依赖库（so文件）无缺失"
flag=0
tmp="tmp"

# 前置处理：应用已安装
function testcase_compatibility01_setup()
{
    deb_check_install_status ${PKG_PACKAGENAME}
    if [ "$?"x == "1"x ];then
        log_info "应用 ${PKG_PACKAGENAME} 未安装，需进行安装"
        deb_install ${PKG_PATH}
        if [ "$?"x == "1"x ];then
            log_error "应用 ${PKG_PACKAGENAME} 安装失败，无法进行软件依赖库（so文件）查询测试"
            flag=1
            # 前置条件1不满足，无需进行前置条件2判断，退出函数
            return
        fi
    fi
    decompressed_package_to_tmp_directory ${tmp}
    if [ "$?"x == "1"x ];then
        log_error "应用${PKG_PACKAGENAME}解压失败，无法进行软件依赖库（so文件）查询测试"
        flag=1
    fi
}

# 后置处理：删除解压应用的临时文件
function testcase_compatibility01_teardown()
{
    delete_tmp_path ${tmp} # 删除临时目录tmp
}

# 测试步骤：依次检查ELF文件所需动态库so文件状态
function testcase_compatibility01_teststep()
{
    if [ "${flag}"x == "0"x ]; then
        elf_list=($(get_elf_list ${tmp})) # 获取应用全部ELF格式文件
        if [[ ${elf_list[*]} == "" ]];then
            flag="NA"
            log_info "应用 ${PKG_PACKAGENAME} 软件不包含ELF格式文件,无需进行软件依赖库（so文件）查询测试"
        else
            check_so_file ${elf_list[*]}
            if [ "$?" == "0" ];then
                log_info "应用${PKG_PACKAGENAME}软件依赖库（so文件）查询测试通过"
            else
                flag="WARNING"
                log_warn "应用${PKG_PACKAGENAME}软件依赖库（so文件）查询测试失败"
            fi
        fi
    fi
}

function TestCase_common_compatibility_01()
{
    write_caseinfo_to_log "${Casenumber}" "${Description}"
    testcase_compatibility01_setup
    testcase_compatibility01_teststep
    testcase_compatibility01_teardown
    write_endinfo_to_log "${Casenumber}"
}

# 执行测试
TestCase_common_compatibility_01 >> "${CASE_LOG_FILE}"
# assert 测试步骤是否全部PASS
assert_result_by_flag "${flag}" "${Casenumber}"