#!/bin/bash
#######################################################################################
#Function: check the version of gcc required by ELF files
#Platform:all
#Author: RongJia Du
#Date: 2022/05/26
#Company: Kylin Software Co.,Ltd.
#Description: 验证应用ELF文件gcc编译器版本是否被系统支持
########################################################################################
source ./kylincompatibilitytest/config/config
source ./kylincompatibilitytest/lib/common_function.sh
source ./config/configuration.conf
source ./pub/baselib.sh
source ./pub/log.sh

Tags="[desktop,compatibility,p0]"
Casenumber="common-compatibility-03"
Casename="软件gcc版本兼容性测试"
Description="[common-compatibility-03]：验证应用所需gcc版本小于等于系统gcc版本"
flag=0
tmp="tmp"

# 前置处理：解压应用全部安装文件到临时目录tmp
function testcase_compatibility03_setup()
{
    decompressed_package_to_tmp_directory ${tmp}
    if [ "$?" = "1" ];then
        log_error "应用${PKG_PACKAGENAME}解压失败，无法进行gcc版本兼容性测试"
        flag=1
    fi
}

# 后置处理：删除解压应用的临时文件
function testcase_compatibility03_teardown()
{
    delete_tmp_path ${tmp} # 删除临时目录tmp
    delete_tmp_file "./kylincompatibilitytest/resource/gcc" # 删除临时文件gcc
}

# 测试步骤：依次检查ELF文件gcc版本信息与系统版本是否匹配
function testcase_compatibility03_teststep()
{
    if [ "${flag}" = "0" ]; then
        elf_list=($(get_elf_list ${tmp})) # 获取应用全部ELF格式文件
        app_gcc=($(get_app_gcc_version ${elf_list[*]})) # 获取应用gcc编译器版本
        if [[ ${app_gcc[*]} == "" ]];then
            flag="NA"
            log_info "应用${PKG_PACKAGENAME}软件未涉及gcc编译器，无需进行gcc版本兼容性测试"
        else
            check_app_gcc_version ${app_gcc[*]}
            if [ "$?" == "0" ];then
                log_info "应用${PKG_PACKAGENAME}所需gcc版本小于等于系统gcc版本"
            else
                flag="WARNING"
                log_warn "该deb包所使用的gcc兼容性测试失败"
            fi
        fi
    fi
}

function TestCase_common_compatibility_03()
{
    write_caseinfo_to_log "${Casenumber}" "${Description}"
    testcase_compatibility03_setup
    testcase_compatibility03_teststep
    testcase_compatibility03_teardown
    write_endinfo_to_log "${Casenumber}"
}

# 执行测试
TestCase_common_compatibility_03 >> "${CASE_LOG_FILE}"
# assert 测试步骤是否全部PASS
assert_result_by_flag "${flag}" "${Casenumber}"