#!/bin/bash
#######################################################################################
#Function: check the version of glibc required by ELF files
#Platform:all
#Author: RongJia Du
#Date: 2022/05/27
#Company: Kylin Software Co.,Ltd.
#Description: 验证应用ELF文件glibc版本是否包含于系统glibc支持版本
########################################################################################
source ./kylincompatibilitytest/config/config
source ./kylincompatibilitytest/lib/common_function.sh
source ./config/configuration.conf
source ./pub/baselib.sh
source ./pub/log.sh

Tags="[desktop,compatibility,p0]"
Casenumber="common-compatibility-04"
Casename="软件glibc版本兼容性测试"
Description="[common-compatibility-04]：验证应用所需glibc版本符合系统支持glibc版本"
flag=0
tmp="tmp"

# 前置处理：应用已解压
function testcase_compatibility04_setup()
{
    decompressed_package_to_tmp_directory ${tmp}
    if [ "$?" = "1" ];then
        log_error "应用${PKG_PACKAGENAME}解压失败，无法进行glibc版本兼容性测试"
        flag=1
    fi
}

# 后置处理：删除解压应用的临时文件
function testcase_compatibility04_teardown()
{
    delete_tmp_path ${tmp} # 删除临时目录tmp
    delete_tmp_file "./kylincompatibilitytest/resource/glibc" # 删除临时文件glibc
}

# 测试步骤：依次检查ELF文件glibc版本信息是否为系统支持版本
function testcase_compatibility04_teststep()
{
    if [ "${flag}" = "0" ]; then
        elf_list=($(get_elf_list ${tmp})) # 获取应用全部ELF格式文件
        app_glibc=($(get_app_glibc_version ${elf_list[*]})) # 获取应用全部glibc版本
        if [[ ${app_glibc[*]} == "" ]];then
            flag="NA"
            log_info "应用${PKG_PACKAGENAME}软件未涉及glibc，无需进行glibc版本兼容性测试"
        else
            check_app_glibc_version ${app_glibc[*]}
            if [ "$?" == "0" ];then
                log_info "应用${PKG_PACKAGENAME}所需glibc版本符合系统支持glibc版本"
            else
                flag="WARNING"
                log_warn "该deb包所使用的glibc兼容性测试失败"
            fi
        fi
    fi
}

function TestCase_common_compatibility_04()
{
    write_caseinfo_to_log "${Casenumber}" "${Description}"
    testcase_compatibility04_setup
    testcase_compatibility04_teststep
    testcase_compatibility04_teardown
    write_endinfo_to_log "${Casenumber}"
}

# 执行测试
TestCase_common_compatibility_04 >> "${CASE_LOG_FILE}"
# assert 测试步骤是否全部PASS
assert_result_by_flag "${flag}" "${Casenumber}"