#!/bin/bash
#######################################################################################
#Function: check kernel object file can install and uninstall correctly
#Platform:amd64\arm64\x86
#Author: Chunli Wang
#Date: 2022/06/16
#Company: Kylin Software Co.,Ltd.
#Description: 验证目标应用内的内核模块文件都能在测试环境中正常加载，卸载
########################################################################################
source ./kylincompatibilitytest/config/config
source ./kylincompatibilitytest/lib/common_function.sh
source ./kylincompatibilitytest/lib/ko_process.sh
source ./config/configuration.conf
source ./pub/baselib.sh
source ./pub/log.sh

Tags="[desktop,compatibility,p0]"
Casenumber="common-compatibility-05"
Casename="内核模块兼容性检查"
Description="[common-compatibility-05]：验证目标应用内的内核模块文件都能在测试环境中正常加载，卸载"
flag=0

CONFIG_PATH=${PROJECT_PATH}/config/config

# 前置处理：判断应用已安装
function testcase_compatibility05_setup()
{
    local drive_path=${PROJECT_PATH}/resource/${PKG_PACKAGENAME}
    local ko_report=${drive_path}/${PKG_PACKAGENAME}_ko_test_results.csv
    sed -i "s|DRIVE_PATH=.*|DRIVE_PATH=${drive_path}|" ${CONFIG_PATH}
    sed -i "s|KO_REPORT=.*|KO_REPORT=${ko_report}|" ${CONFIG_PATH}
    source ${CONFIG_PATH}

    echo "序号,ko文件名,对应架构,测试环境架构,对应内核版本号,测试环境内核版本号,加载,卸载" > ${KO_REPORT}

    log_info "正在检查应用包 ${PKG_PACKAGENAME}"

    get_ko "${PKG_PATH}"
    if [ $? -ne 0 ]; then
        flag="NA"
    fi
}

# 测试步骤：
function testcase_compatibility05_teststep()
{
    if [ "${flag}" == "0" ]; then
        get_ko_info_list
        get_test_ko_list
        # 测试 ko 文件
        IFS=$IFS_old
        test_ko
        IFS=$'\n'

        get_ko_test_result
        if [ "$?" == "0" ];then
            log_info "应用${PKG_PACKAGENAME}的内核模块文件都能正常加载、卸载，具体信息可查看报告${ko_report}"
        else
            flag="WARNING"
            log_warn "应用${PKG_PACKAGENAME}内核模块文件存在不能正常加载、卸载的问题，具体信息可查看报告${ko_report}"
        fi
    fi
}


function testcase_compatibility05_teardown()
{
    log_info "应用包 ${PKG_PACKAGENAME} 检查完毕"
    #sed -i '/DRIVE_PATH=/,$d' ${CONFIG_PATH}
}

function TestCase_common_compatibility_05()
{
    write_caseinfo_to_log "${Casenumber}" "${Description}"
    testcase_compatibility05_setup
    testcase_compatibility05_teststep
    testcase_compatibility05_teardown
    write_endinfo_to_log "${Casenumber}"
}

# 执行测试
TestCase_common_compatibility_05 >> "${CASE_LOG_FILE}"
# assert 测试步骤是否全部PASS
assert_result_by_flag "${flag}" "${Casename}"