#!/bin/bash
#######################################################################################
#Function: update app
#Platform:all
#Author: Fenfen Liu,Chunli Wang
#Date: 2022/03/16
#Company: Kylin Software Co.,Ltd.
#Description: 验证应用能够在麒麟桌面操作系统上成功更新
########################################################################################
source ./kylincompatibilitytest/config/config
source ./kylincompatibilitytest/lib/common_function.sh
source ./config/configuration.conf
source ./pub/baselib.sh
source ./pub/log.sh

Tags="[desktop,function,p1]"
Casenumber="common-function-01"
Casename="应用成功更新"
Description="[common-function-01]：验证应用能够在麒麟桌面操作系统上成功更新"
flag=0

# 前置处理：应用已安装，且版本低于待测deb包
function testcase_function01_setup()
{
    deb_check_install_status ${PKG_PACKAGENAME}
    if [ "$?" = "0" ];then
        version=$(dpkg -l ${PKG_PACKAGENAME} | tail -1 | awk '{print $3}')
        version_max=$(echo "${version} ${PKG_VERSION}" | tr ' ' '\n' | sort -rV | tr '\n' ' ' | awk '{print $1}') # 比较待测应用与已安装应用的版本大小
        if [ "${version}" == "${version_max}" ]; then
            log_info "应用 ${PKG_PACKAGENAME} 已预装，且为最新版本 ${version}，无需升级测试"
            flag="NA"
        fi
    else
        log_info "应用 ${PKG_PACKAGENAME} 软件未安装,无需升级测试"
        flag="NA"
    fi
}

# 测试步骤：更新安装应用
function testcase_function01_teststep()
{
    if [ "${flag}" = "0" ]; then
        deb_install ${PKG_PATH}
        flag=$?
    fi
}

function TestCase_common_function_01()
{
    write_caseinfo_to_log "${Casenumber}" "${Description}"
    testcase_function01_setup
    testcase_function01_teststep
    write_endinfo_to_log "${Casenumber}"
}

# test
TestCase_common_function_01 >> "${CASE_LOG_FILE}"
# assert 测试步骤是否全部PASS
assert_result_by_flag "${flag}" "${Casenumber}"