#!/bin/bash
#######################################################################################
#Function: measure app launch time
#Platform:all
#Author: Chunli Wang
#Date: 2022/03/16
#Company: Kylin Software Co.,Ltd.
#Description: 软件在麒麟桌面操作系统中的启动时长测试
########################################################################################
source ./kylincompatibilitytest/config/config
source ./kylincompatibilitytest/lib/common_function.sh
source ./config/configuration.conf
source ./pub/baselib.sh
source ./pub/log.sh

Tags="[desktop,performance,p1]"
Casenumber="common-performance-01"
Casename="软件在麒麟桌面操作系统上打开的时长正常(应用启动10次，平均启动时长不超过3s)"
Description="[common-performance-01]：验证应用在麒麟桌面操作系统上打开时长正常"
flag=0

# 前置处理：判断应用已安装
function testcase_performance01_setup()
{
    deb_check_install_status xdotool
    if [ "$?" == 1 ];then
        echo "安装麒麟桌面适配测试工具依赖软件xdotool"
        deb_install xdotool
        if [ "$?" == 1 ];then
            log_warn "当前系统仓库源中未包含xdotool应用，请手动安装完毕后重新测试！"
            flag=1
        fi
    fi

    deb_check_install_status ${PKG_PACKAGENAME}
    if [ "$?" = "1" ];then
        log_info "应用 ${PKG_PACKAGENAME} 未安装，需进行安装"
        deb_install ${PKG_PATH}
        if [ "$?" = "1" ];then
            log_error "应用 ${PKG_PACKAGENAME} 安装失败，无法进行重复启动关闭测试"
            flag=1
        fi
    fi
}

# 测试步骤：启动应用
function testcase_performance01_teststep()
{
    if [ "${flag}" = "0" ]; then
        local total_time=0
        local num=0
        for ((i=1;i<=${REPEAT_START_TIMES};i++))
        do
            log_info "应用 ${PKG_PACKAGENAME} 第 $i 次启动"
            start_app ${PKG_PACKAGENAME} "time"
            local res=$?
            if [ ${res} -lt ${STARTUP_WAIT_TIME} ]; then
                ((num++))
                ((total_time+=res))
            fi
        done
        if [ ${num} -eq 0  ]; then
            log_error "==== ${PKG_PACKAGENAME} 启动全部失败或超时 ===="
            flag=1
        else
            local real_time=$(echo "${total_time} ${num} 10" | awk '{printf("%.1f",$1/$2/$3)}')
            log_info "==== ${PKG_PACKAGENAME}成功启动 ${num} 次，平均启动时间为 ${real_time}s ===="
            real_time=$(echo "${real_time} 10" | awk '{printf("%.0f",$1*$2)}')
            if [ ${real_time} -gt ${APP_START_TIME} ]; then
                flag=1
            fi
        fi
    fi
}

function exec_testcase_common_performance_01()
{
    write_caseinfo_to_log "${Casenumber}" "${Description}"
    testcase_performance01_setup
    testcase_performance01_teststep
    write_endinfo_to_log "${Casenumber}"
}

# test
exec_testcase_common_performance_01 >> "${CASE_LOG_FILE}"
# assert 测试步骤是否全部PASS
assert_result_by_flag "${flag}" "${Casenumber}"