#!/bin/bash
#######################################################################################
#Function: start app repeatedly
#Platform:all
#Author: Chunli Wang
#Date: 2022/03/16
#Company: Kylin Software Co.,Ltd.
#Description: 软件在麒麟桌面操作系统中重复启动关闭测试
########################################################################################
source ./kylincompatibilitytest/config/config
source ./kylincompatibilitytest/lib/common_function.sh
source ./config/configuration.conf
source ./pub/baselib.sh
source ./pub/log.sh

Tags="[desktop,reliability,p1]"
Casenumber="common-reliability-01"
Casename="软件在麒麟桌面操作系统中重复启动关闭测试"
Description="[common-reliability-01]：验证软件在麒麟桌面操作系统上的启动情况"
flag=0

# 前置处理：判断应用已安装
function testcase_reliability01_setup()
{
    deb_check_install_status ${PKG_PACKAGENAME}
    if [ "$?" = "1" ];then
        log_info "应用 ${PKG_PACKAGENAME} 未安装，需进行安装"
        deb_install ${PKG_PATH}
        if [ "$?" = "1" ];then
            log_error "应用 ${PKG_PACKAGENAME} 安装失败，无法进行重复启动关闭测试"
            flag=1
        fi
    fi
}

# 测试步骤：启动应用
function testcase_reliability01_teststep()
{
    if [ "${flag}" = "0" ]; then
        for ((i=1;i<=${REPEAT_START_TIMES};i++))
        do
            log_info "应用 ${PKG_PACKAGENAME} 第 $i 次启动"
            start_app ${PKG_PACKAGENAME} "start"
            if [ $? -eq 255 ]; then
                ((flag+=1))
            fi
        done
    fi
}

function exec_testcase_common_reliability_01()
{
    write_caseinfo_to_log "${Casenumber}" "${Description}"
    testcase_reliability01_setup
    testcase_reliability01_teststep
    write_endinfo_to_log "${Casenumber}"
}

# test
exec_testcase_common_reliability_01 >> "${CASE_LOG_FILE}"
# assert 测试步骤是否全部PASS
assert_result_by_flag "${flag}" "${Casenumber}"