#!/bin/bash
#######################################################################################
#Function: function of debcheck
#Platform:amd64\arm64\x86
#Author: Fenfen Liu,Chunli Wang
#Date: 2022/03/16
#Company: Kylin Software Co.,Ltd.
#Description: 确认spec文件中包名、版本号、架构三个字段符合打包规范，包名符合规范，包名中信息与spec文件中对应字段一致
########################################################################################
source ./kylincompatibilitytest/config/config
source ./kylincompatibilitytest/lib/common_function.sh
source ./config/configuration.conf
source ./pub/baselib.sh
source ./pub/log.sh

Tags="[server,base,p0]"
Casenumber="common-base-02"
Casename="包名规范一致性检查"
Description="[common-base-02]：确认spec文件中包名、版本号、架构三个字段符合打包规范，包名符合规范，包名中信息与spec文件中对应字段一致"
flag=0
fail_num=0
# 检查关键字段是否符合字段规范
function check_keyword()
{
    local keyword=$1
    local rule=$2
    local loginfo=$3
    echo "${keyword}" | grep -P "${rule}"
    if [ "$?" = "1" ]; then
        fail_num=$[fail_num+1]
        log_error ${loginfo} >> "${CASE_LOG_FILE}"
    fi
}

# 测试步骤：分步检查包规范
function testcase_base02_teststep()
{
    # step1-检查Package字段是否符合规范
    log_info "spec文件中包名为：${PKG_PACKAGENAME}，匹配规则为：${RPM_PACKAGE_RULE}"
    check_keyword ${PKG_PACKAGENAME} ${RPM_PACKAGE_RULE} "spec文件的Name字段不符合规范"
    # step2-检查Version字段是否符合规范
    log_info "spec文件中版本号为：${PKG_VERSION}，匹配规则为：${RPM_VERSION_RULE}"
    check_keyword ${PKG_VERSION} ${RPM_VERSION_RULE} "spec文件的Version字段不符合规范"
    # step4-检查Release字段是否符合规范
    log_info "spec文件中release版本号为：${PKG_RELEASE}，匹配规则为：${RPM_RELEASE_RULE}"
    check_keyword ${PKG_RELEASE} ${RPM_RELEASE_RULE} "spec文件的Release字段不符合规范"
    # step3-检查Architecture字段是否符合规范
    local architecture=$(rpm -qpi ${PKG_PATH}  | grep -i "^Architecture" | awk -F ":" '{print $2}' | sed -e 's/ //g' )
    log_info "spec文件中架构为：${architecture}，匹配规则为：${RPM_ARCHITECTURE_RULE}"
    check_keyword ${architecture} ${RPM_ARCHITECTURE_RULE} "spec文件的Architecture字段不符合规范"
    # step4-检查rpm_name是否符合规范
    log_info "软件包名称为：${PKG_SOFTNAME}，匹配规则为：${RPM_PACK_RULE}"
    check_keyword ${PKG_SOFTNAME} ${RPM_PACK_RULE} "rpm包的命名不符合规范"
    # step5-包名称（deb_name）与control文件Package、Version、Architecture字段内容一致
    local pkg_name="${PKG_PACKAGENAME}-${PKG_VERSION}-${PKG_RELEASE}.${architecture}.rpm"
    pkg_name=${pkg_name//\+/\\\+}
    pkg_name=${pkg_name//\./\\\.}
    pkg_name="^"${pkg_name}"$"
    log_info "软件名称为：${PKG_SOFTNAME}，匹配规则为：${pkg_name}"
    check_keyword ${PKG_SOFTNAME} ${pkg_name} "rpm包命名字段与spec文件不一致"

    if [ "${fail_num}" = "0" ]; then
        flag=0
    else
        flag=1
    fi
}

function TestCase_common_base_02()
{
    write_caseinfo_to_log "${Casenumber}" "${Description}"
    testcase_base02_teststep
    write_endinfo_to_log "${Casenumber}"
}

# 执行测试
TestCase_common_base_02 >> "${CASE_LOG_FILE}"
# assert 测试步骤是否全部PASS
assert_result_by_flag "${flag}" "${Casenumber}"
