#!/bin/bash
#######################################################################################
#Function: function of base libarary
#Platform:amd64\arm64\x86
#Author: Fenfen Liu
#Date: 2022/1/11
#Company: Kylin Software Co.,Ltd.
#Description:
########################################################################################
source ./pub/baselib.sh
source ./config/configuration.conf
source ./pub/log.sh
source ./kylincompatibilitytest/config/config

# 将用例开始信息写入日志
function write_caseinfo_to_log()
{
    local casenumber=$1
    local case_description=$2
    echo "################## 测试用例 ${casenumber} 开始测试 ##################" >> "${CASE_LOG_FILE}"
    echo "用例描述：${case_description}" >> "${CASE_LOG_FILE}"
}

# 将用例结束信息写入日志
function write_endinfo_to_log()
{
    local casename=$1
    echo "################## 测试用例 $1 测试结束 ##################" >> "${CASE_LOG_FILE}"
}

# 获取应用启动时间
# 参数1：  启动命令
# 参数2：  测试模式:启动——start，启动时长——time
# 结果：   启动时长
# 返回值：0——启动成功，1-100——启动时长，255——启动失败
function startup() {

    # 整体过程
    # 1.通过对比命令启动前后的进程对比获取命令相关进程
    # 2.在命令启动成功，找到相关进程的情况下，根据以上进程寻找对应窗口，计算启动时间

    local cmd=$1
    local test_mode=$2
    local flag=255
    log_info "==== 验证启动 ${cmd} ===="
    ps | awk '{print $1,$NF}' | grep -v -E ' bash$| sh$| ps$| awk$| grep$| cat$| ukui-screensave' > before.log
    cp before.log last_after.log
    # 判断命令是否带有环境变量
    if [[ "${cmd}" =~ [A-Z_]+= ]]; then
        log_info "===== 命令中带有环境变量 ====="
        cat > exec.sh << EOF
#!/bin/bash
${cmd} > /dev/null 2>&1 &
EOF
        echo ${PASSWD} | sudo -S chmod 777 exec.sh
        ./exec.sh > /dev/null 2>&1 &
    else
        log_info "===== 命令中不带环境变量 ====="
        ${cmd} > /dev/null 2>&1 &
    fi
    local cmd_time=1
    # 每1.3s查一次进程，最长等待时间20s
    while [ ${cmd_time} -lt ${SLEEP_TIME} ];
    do
        ps | awk '{print $1,$NF}' | grep -v -E ' bash$| sh$| ps$| awk$| grep$| cat$| ukui-screensave' > after.log
        local add_process=$(diff before.log after.log | grep -E "^>")
        local app_process_id=($(diff before.log after.log | grep -E "^>" | awk -F " " '{print $2}' | sed 's/\ /\n/g'))
        local app_process_number=$(diff before.log after.log | grep -E "^>" | awk -F " " '{print $2}' | wc -l)
        local last_process_number=$(diff last_after.log after.log | grep -E "^>" | awk -F " " '{print $2}' | wc -l)

        if [ ${app_process_number} -gt 0 ]; then
            log_info "获取到新增进程 ${add_process[*]}"
            log_info "获取到新增进程 id为：${app_process_id[*]}"
            log_info "较开始新增进程数量 ${app_process_number}"
            log_info "较前1.3s新增进程数量 ${last_process_number}"
        fi

        # before.log保存命令执行前的进程，last_after.log保存上一个1.3s的进程，和after.log保存本次的进程
        # 当新增进程稳定时测试启动时间，当before.log和after.log有差别而last_after.log和after.log没有差别时为进程稳定，代价是多测了0.3s，保证了不会过早进入下一个阶段而漏掉一些新增进程
        if [ ${app_process_number} -gt 0 -a ${last_process_number} -eq 0 ];then
            log_info "==== ${cmd}启动成功 ===="
            if [ "${test_mode}" == "start" ]; then
                flag=0
            elif [ "${test_mode}" == "time" ]; then
                local wids=""
                local time=0

                # 通过上面获取的进程，查找对应窗口，判断当前最前窗口是否在命令相关进程对应的窗口集合中，是则认为启动完成，停止计时，否则继续等待
                # 每0.1s查询一次，最长等待时间10s
                while [ ${time} -lt ${STARTUP_WAIT_TIME} ]
                do
                    local wids=""
                    for id in ${app_process_id[*]}
                    do
                        local new_wids=$(xdotool search --pid "${id}")
                        if [ "${new_wids}"x != ""x ]; then
                            if [ "${wids}"x = ""x ]; then
                                wids=${new_wids}
                            else
                                wids=${wids}" "${new_wids}
                            fi
                        fi
                    done
                    echo ${wids}
                    local active_wid=$(xdotool getactivewindow)
                    if [ "$(echo ${wids} | grep ${active_wid})"x != ""x ]; then
                        log_info "获取到对应窗口 ${wids},当前窗口为 ${active_wid}"
                        break
                    fi
                    sleep 0.1
                    ((time+=1))
                done
                flag=${time}
            fi
            break
        else
            # 修改dzh（大智慧）启动闪退问题，将进程检测间隔拉长到1.3s
            sleep 1.3
            ((cmd_time+=13))
            cp after.log last_after.log
        fi
    done

    if [ ${flag} -ge 0 -a ${flag} -le ${STARTUP_WAIT_TIME} ]; then
        kill_pid "${app_process_id[*]}"
        if [ ${flag} -ne 0 ]; then
            local real_time=$(echo "${flag} 10" | awk '{printf("%.1f",$1/$2)}')
            if [ ${flag} -eq ${STARTUP_WAIT_TIME} ]; then
                log_warn "==== 本次启动已超时，达到最长等待时间 ${real_time}s ===="
            else
                log_info "==== 本次启动时长为 ${real_time}s ===="
            fi
        fi
    else
        log_error "==== ${cmd}启动失败 ===="
    fi

    rm -f after.log
    rm -f last_after.log
    rm -f before.log
    rm -f exec.sh

    return ${flag}
}

## 获取应用启动时间
## 参数1：  启动命令
## 参数2：  测试模式
## 结果：   启动时长
## 返回值：0——启动成功，1-100——启动时长，255——启动失败
#function startup() {
#    local cmd=$1
#    local test_mode=$2
#    log_info "==== 验证启动 ${cmd}===="
#    ${cmd} &
#    local main_cmd=$(echo "${cmd}" | awk '{print $1}')
#    if [ "$(file "${main_cmd}" | grep shell)"x != ""x ]; then
#        local fpid=$(ps -ef | grep "${cmd}" | grep -v root | grep -v grep | head -1 | awk '{print $2}')
#        local pid=$(ps -ef | grep ${fpid} | grep -v root | grep -v grep | head -2 | tail -1 | awk '{print $2}')
#    else
#        local pid=$(ps -ef | grep "${cmd}" | grep -v root | grep -v grep | head -1 | awk '{print $2}')
#        fi
#    if [ "${pid}x" != ""x ]; then
#        log_info "获取到应用进程 ${pid}"
#        log_info "==== ${cmd}启动成功 ===="
#        if [ "${test_mode}" == "start" ]; then
#            echo ${PASSWD} | sudo -S kill -9 ${pid}
#            return 0
#        elif [ "${test_mode}" == "time" ]; then
#            local wids=""
#            local time=0
#            while [ ${time} -lt ${STARTUP_WAIT_TIME} ]
#            do
#                wids=$(xdotool search --pid ${pid})
#                local active_wid=$(xdotool getactivewindow)
#                if [ "$(echo ${wids} | grep ${active_wid})"x != ""x ]; then
#                    log_info "获取到对应窗口 ${wids},当前窗口为 ${active_wid}"
#                    break
#                fi
#                sleep 0.1
#                ((time+=1))
#            done
#            echo ${PASSWD} | sudo -S kill -9 ${pid}
#            local real_time=$(echo "${time} 10" |awk '{printf("%.1f",$1/$2)}')
#            if [ ${time} -eq ${STARTUP_WAIT_TIME} ]; then
#                log_warn "==== 本次启动已超时，达到最长等待时间 ${real_time}s ===="
#            else
#                log_info "==== 本次启动时长为 ${real_time}s ===="
#            fi
#            return ${time}
#        fi
#    else
#        log_error "==== ${cmd}启动失败 ===="
#        return 255
#    fi
#}

## 验证启动
#function process_startup() {
#	local cmd=$1
#	log_info "==== 验证启动 ${cmd}===="
#	ps | awk '{print $1,$NF}' > before.log
#	${cmd} &
#	sleep ${SLEEP_TIME}
#	ps | awk '{print $1,$NF}' > after.log
#	local add_process=`diff before.log after.log |grep \>`
#	log_info "获取到新增进程 ${add_process}"
#	local app_process_id=($(diff before.log after.log |grep \> |awk -F " " '{print $2}'))
#	log_info "获取到新增进程 id为：${app_process_id[*]}"
#	local app_process_number=`diff before.log after.log |grep \> |awk -F " " '{print $2}' | wc -l`
#	log_info "新增进程数量 ${app_process_number}"
#	if [ $app_process_number -gt 2 ];then
#		log_info "==== ${cmd}启动成功 ===="
#		kill_pid "${app_process_id[*]}"
#	else
#		start_failnum=1
#		log_error "==== ${cmd}启动失败 ===="
#	fi
#	rm -f after.log
#	rm -f before.log
#}

# 处理启动命令格式并执行测试
# 入参1:  启动命令
# 入参2：  测试模式
# 返回值：0——启动成功，1-100——启动时长，255——启动失败，254——不符合测试条件，未测试
function exec_name() {
    local cmd="$1"
    local test_mode="$2"
    # 检查命令是否含有停止关闭等关键字
    if [[ "${cmd}" =~ --(shutdown|stop)$ ]]; then
        log_warn "==== ${cmd}为关闭命令，不测试 ===="
        return 254
    else
        startup "${cmd}" "${test_mode}"
        local res=$?
        if [[ ${res} -eq 255 ]];then
            # 原始exec命令启动失败的情况下，去掉Exec的%U、%F等字段进行启动
            if [[ "${cmd}" =~ % ]];then
                cmd=$(echo "${cmd}" | sed 's/ %[A-Za-z]//g')
                log_info "==== 去掉命令参数重试 ===="
                startup "${cmd}" "${test_mode}"
                res=$?
            fi
            # 原始exec命令启动失败、去掉Exec的%U、%F等字段启动失败情况下，处理Exec的特殊字符尝试启动？？？？？？
            if [[ ${res} -eq 255 ]] && [[ "${cmd}" =~ \" ]];then
                cmd="$(echo "${cmd}" | sed 's/\"//g')"
                log_info "==== 去掉命令中引号重试 ===="
                startup "${cmd}" "${test_mode}"
                res=$?
            fi
        fi
        return ${res}
    fi
}

# 测试desktop文件中的启动方式
# 参数1：desktop文件列表
# 参数2：测试项目代码，1——启动，2——启动时间
# 返回值:0——启动成功，1-100——启动时长，255——启动失败
function exec_desktop() {
    local desktop_list=$1
    local test_mode=$2
    local start_success_num=0
    local start_time_sum=0
    local start_fail_num=0
    log_info "=========== 获取启动方式 ==========="

    if [ -e ${desktop_list} ]; then
        if [ -f ${desktop_list} ]; then
            while read desktop
            do
                if [ -e ${desktop} ]; then
                    local desktop_title=$(cat ${desktop} | grep "\[Desktop" | wc -l)
                    # desktop文件中存在Desktop Entry和其他字段，以Desktop Entry的启动方式Exec为准
                    if [ "${desktop_title}" -gt '1' ];then
                        local exec_key=$(cat "${desktop}" | grep -zPo '\[Desktop Entry\][\w\W]+?(?=\[Desktop)' | grep -ai "^\s*Exec" | sed "s/\s*Exec\s*=\s*//g" | sed 's/\r//g')
                        echo ${exec_key} >> exec.tmp
                    else
                        local exec_key=$(cat ${desktop} | grep -ai "^\s*Exec" | sed "s/\s*Exec\s*=\s*//g" | sed 's/\r//g')
                        echo ${exec_key} >> exec.tmp
                    fi
                    log_info "启动文件 ${desktop}的启动方式为：${exec_key}"
                else
                    log_error "启动文件 ${desktop} 未找到"
                fi
            done < ${desktop_list}

            if [ -s exec.tmp ]; then
                log_info "=========== 开始启动测试 ==========="
                cat exec.tmp | sort | uniq > exec.tmp2

                while read exec_line
                do
                    exec_name "${exec_line}" "${test_mode}"
                    local res=$?
                    if [ ${res} -gt ${STARTUP_WAIT_TIME} ]; then
                        ((start_fail_num+=1))
                    else
                        ((start_success_num++))
                        ((start_time_sum+=res))
                    fi
                done < exec.tmp2

                rm -f exec.tmp
                rm -f exec.tmp2
            else
                log_error "未找到任何启动方式，退出测试"
                return 255
            fi
        else
            ((start_fail_num+=1))
            log_error "${desktop_list} 不是文件"
        fi
    else
        ((start_fail_num+=1))
        log_error "文件 ${desktop_list} 不存在"
    fi

    if [ "${test_mode}"x == "start"x ]; then
        if [ ${start_fail_num} -gt 0 ]; then
            return 255
        else
            return 0
        fi
    elif [ "${test_mode}"x == "time"x ]; then
        if [ ${start_success_num} -gt 0 ]; then
            local start_time=$(echo "${start_time_sum} ${start_success_num}" |awk '{printf("%.0f",$1/$2)}')
            local real_time=$(echo "${start_time} 10" |awk '{printf("%.1f",$1/$2)}')
            if [ ${start_time} -eq ${STARTUP_WAIT_TIME} ]; then
                log_warn "==== 应用本次启动超时，达到最长等待时间 ${real_time}s ===="
            else
                log_info "==== 本次启动时长为 ${real_time}s ===="
            fi
            return ${start_time}
        else
            return 255
        fi
    fi
}

# 执行shell启动脚本
# 参数1：shell文件列表
# 参数2：测试项目代码，1——启动，2——启动时间
# 返回值:0——启动成功，1-100——启动时长，255——启动失败
function exec_shell() {
    local shell_list=$1
    local test_mode=$2
    local start_success_num=0
    local start_fail_num=0
    local start_time_sum=0

    log_info "查找到启动文件列表: $(cat desktop.tmp)"
    while read shell
    do
        exec_name "${shell}" "${test_mode}"
        local res=$?
        if [ ${res} -gt ${STARTUP_WAIT_TIME} ]; then
            ((start_fail_num+=1))
        else
            ((start_success_num+=1))
            ((start_time_sum+=res))
        fi
    done < ${shell_list}

    if [ "${test_mode}"x == "start"x ]; then
        if [ ${start_fail_num} -gt 0 ]; then
            return 255
        else
            return 0
        fi
    elif [ "${test_mode}"x == "time"x ]; then
        if [ ${start_success_num} -gt 0 ]; then
            local start_time=$(echo "${start_time_sum} ${start_success_num} 10" | awk '{printf("%.1f",$1/$2/$3)}')
            log_info "==== 应用本次启动时长为 ${start_time} ===="
            start_time=$((start_time*10))
            return ${start_time}
        else
            return 255
        fi
    fi
}

# 启动应用
# 参数1：应用名称
# 参数2：测试项目代码，start——启动，time——启动时间
# 返回值:0——启动成功，1-100——启动时长，254——未找到desktop文件，255——启动失败
function start_app()
{
    local package=$1
    local test_mode=$2
    local start_fail=0
    local res=0
    log_info "${package} 查找启动文件"
    log_info "=========获取 ${package} 的启动文件 ========="
    local desk=`dpkg -L ${package} | grep -i '\.desktop$'`
    local desktop_sh=`dpkg -L ${package} | grep desktop.sh`
    if [ -z "$desk" ] && [ -z "$desktop_sh" ]; then
        log_warn "此应用无desktop文件，无法通过脚本启动，请手动检查"
        log_warn "桌面应用必需desktop文件，请按照规范打包"
        return 254
    else
        if [ -n "$desk" ]; then
            dpkg -L ${package} | grep -i '\.desktop$' | grep -v uninstall | grep -v template > desktop.tmp
            log_info "查找到启动文件列表: $(cat desktop.tmp)"
            # 查询开始菜单下的启动路径
            cat desktop.tmp | grep "^/usr/share/applications/" > menu_desktop.tmp
            if [ -n "$(cat menu_desktop.tmp)" ]; then
                log_info "优先启动开始菜单栏的启动文件:$(cat menu_desktop.tmp)"
                exec_desktop "./menu_desktop.tmp" "${test_mode}"
                res=$?
                if [ ${res} -eq 255 ]; then
                    start_fail=1
                fi
            else
                log_warn "开始菜单栏无启动文件，应用desktop文件应放在/usr/share/applications目录下，deb包打包不规范，请检查"
                start_fail=1
            fi
        else
            for file in "${desktop_sh}"
            do
                echo ${file} > desktop.tmp
            done
            exec_shell "./desktop.tmp" "${test_mode}"
            res=$?
            if [ ${res} -eq 255 ]; then
                start_fail=1
            fi
        fi
        rm -f desktop.tmp
        rm -f menu_desktop.tmp
    fi

    if [ ${start_fail} -eq 0 ];then
        log_info "${package} 启动成功"
        return ${res}
    else
        log_error "${package} 启动失败"
        return 255
    fi
}

# 杀灭新增进程
function kill_pid()
{
    local pid_array=($*)
    for pid in ${pid_array[@]}
    do
        log_info "杀死进程 ${pid}"
        echo ${PASSWD} | sudo -S kill -9 $pid > /dev/null 2>&1 &
    done
}

# 处理dpkg锁异常
# 入参（param1：字符串（应用包名）；
function handle_dpkg_exception()
{
    local packagename=$1
    echo ${PASSWD} | sudo -S dpkg --configure -a
    echo ${PASSWD} | sudo -S dpkg --remove --force-remove-reinstreq ${packagename}
}

# 判断case的flag数值，返回case执行结果
# 入参：${flag} ${casenumber}
function assert_result_by_flag()
{
    if [ "$1" = "0" ]; then
        echo "################ 测试用例 $2 测试结果 PASS ################" >> "${CASE_LOG_FILE}"
        return 0
    elif [ "$1" = "1" ]; then
        echo "################## 测试用例 $2 测试结果 FAIL ##################" >> "${CASE_LOG_FILE}"
        return 1
    elif [ "$1" = "NA" ]; then
        echo "################## 测试用例 $2 不进行测试 ##################" >> "${CASE_LOG_FILE}"
        return 2
    elif [ "$1" = "WARNING" ]; then
        echo "################## 测试用例 $2 存在兼容性问题 ##################" >> "${CASE_LOG_FILE}"
        return 3
    else
        echo "################## 测试用例 $2 测试异常 ##################" >> "${CASE_LOG_FILE}"
        return 4
    fi
}

# 获取应用elf文件列表
# 入参：应用解压临时文件目录
function get_elf_list()
{
    local sys_arch=$(arch)
    if [[ "${sys_arch}"x == "x86_64"x ]]; then
        sys_arch="x86-64"
    elif [[ "${sys_arch}"x =~ "mips64"x ]]; then
        sys_arch="MIPS"
    elif [[ "${sys_arch}"x == "loongarch64"x ]]; then
        sys_arch="unknown arch"
    elif [[ "${sys_arch}"x == "sw_64"x ]];then
        sys_arch="Sw_64"
    fi
    find $1 -type f -exec file {} \; | grep "\<ELF\>" | grep -E "${sys_arch}" | awk -F ':' '{print $1}' > tmp_elf # 查找当前应用所有elf文件输出到临时文件tmp_elf中
    local elf_list=($(cat tmp_elf | sed 's/\n/ /g'))
    rm -rf tmp_elf
    echo ${elf_list[*]}
}

# 功能：检查应用ELF文件所需动态库so文件是否有缺失
# 入参：elf文件列表
function check_so_file()
{
    local elf_list=$*
    local file=""
    local so_file=""
    local flag=0
    for i in ${elf_list[@]}
    do
        file=${i/tmp/ } # 删除临时文件tmp，确保查找已安装在系统中的应用文件
        so_file=($(ldd ${i} | grep "not found" | awk '{FR="=>"}{print $1}')) # 获取可执行文件缺失的动态依赖库文件信息
        if [[ ${so_file} != "" ]];then
            log_warn "应用${PKG_PACKAGENAME}ELF文件${file}所需动态依赖库文件有缺失，具体缺失的动态依赖库包括：${so_file[*]}，请人工确认功能影响范围"
            flag=1
        fi
    done
    return ${flag}
}

# 获取应用依赖字段数组
# 入参：无
# 出参：应用依赖字段列表
function get_depends_list()
{
    local depends=$(dpkg -f ${PKG_PATH} Depends) # 获取应用Depends字段
    local depends_string=${depends// /} # 去除Depends多余空格
    local depends_list=(${depends_string//,/ }) # 将,分隔符修改为空格
    echo ${depends_list[*]}
}

# 功能：检查应用依赖字段是否符合兼容性规则
# 入参：应用Depends字段列表
function check_depends_list()
{
    local depends_list=$*
    local error_depends=()
    local compatibility_01=""
    local compatibility_02=""
    local t=0

    log_info "应用${PKG_PACKAGENAME}包含的Depends字段：${depends_list[*]}"
    for i in ${depends_list[@]}
    do
        compatibility_01=$(echo ${i} | grep -E \= | grep -v \>) # 匹配包含=号且不包含>号的字符
        compatibility_02=$(echo ${i} | grep -E [\<]) # 匹配字符串中包含<号
        if [[ ${compatibility_01} != "" || ${compatibility_02} != "" ]];then
            error_depends[t]=$i
            t=$[t+1]
        fi
    done
    # 检查是否包含有兼容性问题depends字段
    if [[ ${error_depends[*]} == "" ]];then
        return 0
    else
        log_warn "应用${PKG_PACKAGENAME}软件依赖字段不符合兼容性规则，具体包括：${error_depends[*]}"
        return 1
    fi
}

# 获取应用文件gcc版本
# 入参：应用elf文件信息
function get_app_gcc_version()
{
    local elf_list=$*
    local app_gcc=()
    for i in ${elf_list[@]}
    do
        gcc=($(strings ${i} | grep -i ^GCC: | awk '{print $NF}'))
        file=${i/${tmp}/.}
        if [[ ${gcc[*]} != "" ]];then
            echo "${file}:${gcc[*]}" >> ./kylincompatibilitytest/resource/gcc
            app_gcc=($(echo ${app_gcc[*]} ${gcc[*]}))
        fi
    done
    app_gcc=($(echo ${app_gcc[*]} | sed 's/ /\n/g' | sort | uniq))
    echo ${app_gcc[*]}
}

# 功能：检查应用所需gcc版本是否被系统支持
# 入参：应用gcc版本
function check_app_gcc_version()
{
    local app_gcc=$*
    local os_gcc=$(gcc --version | grep gcc | awk '{print $NF}'| sed 's/)//g') # 获取系统gcc编译器版本
    local version_max=""
    local file_name=""
    local flag=0
    log_info "应用${PKG_PACKAGENAME}软件gcc编译器版本为：${app_gcc[*]}"
    log_info "当前系统gcc编译器版本为：${os_gcc}"
    for i in ${app_gcc[@]}
    do
        version_max=$(echo "${i} ${os_gcc}" | tr ' ' '\n' | sort -rV | tr '\n' ' ' | awk '{print $1}') # 获取应用gcc与系统gcc版本中较大的版本
        if [[ ${version_max} == ${i} && ${version_max} != ${os_gcc} ]];then # 如果应用gcc版本大于系统gcc版本
            file_name=($(cat ./kylincompatibilitytest/resource/gcc | grep $i | awk -F: '{print $1}')) # 获取包含系统不支持gcc版本的应用文件

            log_warn "系统不支持gcc版本为${i}，涉及文件为：${file_name[*]}，请人工确认功能影响范围"
            flag=1
        fi
    done
    return ${flag}
}

# 获取应用文件glic版本
function get_app_glibc_version()
{
    local elf_list=$*
    local app_glibc=()
    for i in ${elf_list[@]}
    do
        glibc=($(strings $i | grep -i ^GLIBC_))
        file=${i/${tmp}/.}
        echo "${file}:${glibc[*]}" >> ./kylincompatibilitytest/resource/glibc
        app_glibc=($(echo ${app_glibc[*]} ${glibc[*]}))
    done
    app_glibc=($(echo ${app_glibc[*]} | sed 's/ /\n/g' | sort | uniq))
    echo ${app_glibc[*]}
}

# 功能：检查应用glibc版本是否包含与系统glibc版本
# 入参：应用glibc版本列表
function check_app_glibc_version()
{
    local app_glibc=$*
    local os_glibc_file=$(lsof -p $$ 2>&1 | grep "libc-" | awk '{print $NF}') # 找到os中glibc.so文件位置
    local os_glibc=($(strings ${os_glibc_file} | grep ^GLIBC_)) # 获取当前glibc支持的版本
    local check_glibc=()

    log_info "应用${PKG_PACKAGENAME}依赖的glibc版本有：${app_glibc[*]}"
    log_info "当前系统支持的glibc版本有：${os_glibc[*]}"

    # 创建临时文件，用于数据比对
    echo ${app_glibc[*]} | sed 's/ /\n/g' >> app_glibc
    echo ${os_glibc[*]} | sed 's/ /\n/g' >> os_glibc
    check_glibc=($(cat app_glibc os_glibc os_glibc | sort | uniq -u)) # 获取os不支持的应用glibc版本信息
    # 删除临时文件
    rm -rf app_glibc
    rm -rf os_glibc
    if [[ "${check_glibc}" != "" ]];then #如果获取到os不支持的glibc版本
        for i in ${check_glibc[@]}
        do
            file=($(cat ./kylincompatibilitytest/resource/glibc | grep $i | awk -F: '{print $1}')) # 依次获取系统不支持的glibc版本对应的文件信息
            log_warn "应用文件${file[*]}，使用系统不支持的glibc版本${i}，请人工确认功能影响范围"
        done
        return 1
    else
        return 0
    fi
}

# 删除临时文件
# 入参：临时文件名称
function delete_tmp_file()
{
    local file_name=$1
    if [[ -f ${file_name} ]];then
        rm -rf ${file_name}
    fi
}

# 删除临时目录
# 入参：临时目录名称
function delete_tmp_path()
{
    local path_name=$1
    if [[ -d ${path_name} ]];then
        rm -rf ${path_name}
    fi
}

# 将应用包解压到临时文件中
# 入参：临时目录名
function decompressed_package_to_tmp_directory()
{
    local tmp_path=$1
    if [ "${SYSTEM}"x = "desktop"x ];then
        dpkg -x ${PKG_PATH} ${tmp_path}
        local ret=$?
    elif [ "${SYSTEM}"x = "server"x ];then
        mkdir ${tmp_path}
        cd ${tmp_path}
        rpm2cpio ${PKG_PATH} | cpio -idmu > /dev/null
        local ret=$?
        cd - > /dev/null
    fi
    if [ "$?" = "0" ]; then
        return 0
    else
        return 1
    fi
}
