#! /bin/bash

################################################################################
#Function: get drive files from drive packages
#Platform: arm64/amd64/mips64el
#Author: Wang Chunli
#Date: 2022/02/23
#Company: Kylin Software Co.,Ltd.
################################################################################
source ./kylincompatibilitytest/config/config
source ./pub/baselib.sh
source ./pub/log.sh

# 参数列表，1，写入内容；2，写入文件
function write_report() {
    sed -i '$s#$#'"$1"'#g' $2
}

# 获取ko文件
# 入参：应用包路径
function get_ko() {
    local deb="$1"
    local file=$(dpkg -c "$1" | grep -v '\->' | grep -E '\.ko(_kylin.*)?$' | awk '{printf $NF}')

    if [ -z ${file} ]; then
        log_warn "该应用包不含 ko 文件"
        echo "1,无" >> ${KO_REPORT}
        return 1
    else
        # 新建系统文件夹
        local drive_file_name="${DRIVE_PATH}"/ko.txt
        echo ${file} > ${drive_file_name}
        sed -i 's/\.\//\n\//g'  ${drive_file_name}
        sed -i '1d' ${drive_file_name}
        log_info "发现 ko 文件"
			
        # 拿到ko文件
        local drive_files="${DRIVE_PATH}"/"ko"
        mkdir ${drive_files}
        mkdir tmp
        dpkg -x "${deb}" tmp
        while read line
        do
            cp tmp/${line} ${drive_files}
        done <  ${drive_file_name}
        rm -rf tmp
        return 0
    fi
}

# 查询ko文件的某个信息
# 参数1：ko文件
# 参数2：字段名
function get_ko_info() {
    local ko=$1
    case $2 in
    "name")
        local result=$(modinfo ${DRIVE_PATH}/ko/${ko} | grep -E '^name' | awk -F : '{print $2}' | xargs )
        ;;
    "arch")
        local result=$(modinfo ${DRIVE_PATH}/ko/${ko} | grep vermagic | awk -F : '{print $2}' | awk '{print $NF}')
        ;;
    "version")
        local result=$(modinfo ${DRIVE_PATH}/ko/${ko} | grep vermagic | awk -F : '{print $2}' | awk '{print $1}')
        ;;
    esac

    if [ "${result}"x == ""x ]; then
        write_report ",NA" ${DRIVE_PATH}/ko_info.csv
    else
        write_report ",${result}" ${DRIVE_PATH}/ko_info.csv
    fi
}

# 记录每个ko文件的模块名和对应内核版本
function get_ko_info_list() {
    local ko_lists=$(ls ${DRIVE_PATH}/ko)
    echo "序号,模块,模块名,模块架构,模块对应内核版本" > ${DRIVE_PATH}/ko_info.csv
    local num=1
    log_info "提取ko文件信息"
    for ko in ${ko_lists}
    do
        echo "${num},${ko}" >> ${DRIVE_PATH}/ko_info.csv
        get_ko_info "${ko}" "name"
        get_ko_info "${ko}" "arch"
        get_ko_info "${ko}" "version"
        ((num++))
    done
    log_info "提取ko文件信息完成，文件位置为 ${DRIVE_PATH}/ko_info.csv"
}

# 获取测试ko列表，对ko文件列表去重
function get_test_ko_list() {
    local name_list=$(cat ${DRIVE_PATH}/ko_info.csv | sed '1d' | awk -F , '{print $3}' | sort -u | grep -v NA)
    > ${DRIVE_PATH}/test_ko_list
    > ${DRIVE_PATH}/error_test_ko_list
    for module_name in ${name_list}
    do
        version_list=$(cat ${DRIVE_PATH}/ko_info.csv | grep ${module_name} | awk -F , '{print $NF}' | grep -v NA)
        # 同名模块如果有带版本号的，则取最新版；否则都写入待测列表
        if [ "${version_list}"x != ""x ]; then
            # 比较内核版本
            # 版本号大致格式主要有三种，分别为：
            # x.x.xxx-两位小版本号
            # x.x.xxx-日期
            # x.x.xxx-其他
            # 目前只比较x.x.xxx部分

            local max_version=$(echo ${version_list} | tr ' ' '\n' | sort -rV | head -1)
            local ko=$(cat ${DRIVE_PATH}/ko_info.csv | grep ${module_name} | grep ${max_version} | awk -F ',' '{print $2}')
            log_info "筛选出模块 ${module_name} 的最高版本为 ${max_version}，对应 ko 文件为 ${ko}"
            echo ${ko} >> ${DRIVE_PATH}/test_ko_list
        else
            # 暂未发现此种情况
            log_info "模块 ${module_name} 无版本号"
            cat ${DRIVE_PATH}/ko_info.csv | grep ${module_name} | awk '{print $2}' >> ${DRIVE_PATH}/test_ko_list
        fi
    done
    log_info "得到内核模块待测列表 ${DRIVE_PATH}/test_ko_list"
    cat ${DRIVE_PATH}/test_ko_list
    # 没有名字的模块最后处理
    local error_ko=$(cat ${DRIVE_PATH}/ko_info.csv | awk -F , '{print $2,$3}' | grep 'NA' | awk '{print $1}')
    if [ "${error_ko}"x != ""x ]; then
        log_warn "将无法测试的ko文件（无模块名）记入问题文件列表 ${DRIVE_PATH}/error_test_ko_list"
        echo "${error_ko}" | tee -a ${DRIVE_PATH}/error_test_ko_list
    fi
}

# 检查ko文件
function test_ko() {
    local index=1
    while read ko
    do
        echo "${index},${ko}" >> ${KO_REPORT}
        log_info "##### ${ko} 测试开始 #####"
        local ko_arch=$(cat ${DRIVE_PATH}/ko_info.csv | grep ${ko} | awk -F , '{print $4}') # 获取ko文件的架构
        write_report ",${ko_arch},$(arch)" ${KO_REPORT}

        local kernel_version=$(cat ${DRIVE_PATH}/ko_info.csv | grep ${ko} | awk -F , '{print $NF}')
        write_report ",${kernel_version},$(uname -r)" ${KO_REPORT}

        install_ko ${DRIVE_PATH}/ko/${ko}
        if [ $? -eq 0 ]; then
            write_report ",PASS" ${KO_REPORT}
        # 如果ko文件已被加载
        elif [ $? -eq 2 ]; then
            write_report "NA" ${KO_REPORT}
        else
            write_report ",FAIL" ${KO_REPORT}
        fi

        uninstall_ko ${DRIVE_PATH}/ko/${ko}
        if [ $? -eq 0 ]; then
            write_report ",PASS" ${KO_REPORT}
        # 如果ko文件未被加载
        elif [ $? -eq 2 ]; then
            write_report "NA" ${KO_REPORT}
        else
            write_report ",FAIL" ${KO_REPORT}
        fi
        ((index++))
        log_info "##### ${ko} 测试完毕 #####"
    done < ${DRIVE_PATH}/test_ko_list
    rm -f results/test_ko_list
}

# 检查ko文件是否已加载到内核
# 入参：ko模块名
function check_ko_loading_status() {
    local ko_name="$1"
    ret=$(lsmod | grep ${ko_name})
    if [ "${ret}"x != ""x ]; then
        log_info "$1 已加载到内核"
        return 0
    else
        log_info "$1 未加载到内核"
        return 1
    fi
}

# 判断ko加载是否成功
# 入参：ko文件
function install_ko() {
    local ko="$1"
    local ko_file=${ko##*/}
    local ko_file_name=$(cat ${DRIVE_PATH}/ko_info.csv | grep ${ko_file} | awk -F , '{print $3}')
    check_ko_loading_status "${ko_file_name}"

    if [ $? -eq 0 ]; then
        log_info "${ko_file} 已被加载，不具备测试条件" | tee -a ${LOG}
        return 2
    else
        log_info "测试加载 ${ko_file}"
        echo ${PASSWD} | sudo -S insmod ${ko}
        local ret1=$?
        check_ko_loading_status "${ko_file_name}"
        local ret2=$?
        if [ ${ret1} -eq 0 ] && [ ${ret2} -eq 0 ]; then
            log_info "${ko_file_name} 加载成功"
            return 0
        else
            log_info "${ko_file_name} 加载失败"
            return 1
        fi
    fi
}

# 判断ko卸载是否成功
# 入参：ko文件
function uninstall_ko() {
    local ko="$1"
    local ko_file=${ko##*/}
    local ko_file_name=$(cat ${DRIVE_PATH}/ko_info.csv | grep ${ko_file} | awk -F , '{print $3}')
    check_ko_loading_status "${ko_file_name}"
    if [ $? -eq 1 ]; then
        log_info "${ko_file} 未被加载，不具备测试条件"
        return 2
    else
        log_info "测试卸载 ${ko_file}"
        echo ${PASSWD} | sudo -S rmmod ${ko_file_name}
        local ret1=$?
        check_ko_loading_status "${ko_file_name}"
        local ret2=$?
        if [ ${ret1} -eq 0 ] && [ ${ret2} -eq 1 ]; then
            log_info "${ko_file_name} 卸载成功"
            return 0
        else
            log_info "${ko_file_name} 卸载失败"
            return 1
        fi
    fi
}

# 汇总测试结果
# 入参：无
# 返回值：1：测试失败，0：测试成功
function get_ko_test_result() {
    local fail_num=0
    local pass_num=0
    while read line
    do
        local ko_name=$(echo ${line} | awk -F ',' '{print $2}')
        local insmod_result=$(echo ${line} | awk -F ',' '{print $7}')
        local rmmod_result=$(echo ${line} | awk -F ',' '{print $8}')

        if [ "${insmod_result}" == "FAIL" ] || [ "${rmmod_result}" == "FAIL" ]; then
            fail_num=$[ ${fail_num}+1 ]
            local msg="${ko_name}:"
            if [ "${insmod_result}" == "FAIL" ]; then
                msg=${msg}" 加载失败"
            elif [ "${rmmod_result}" == "FAIL" ]; then
                msg=${msg}" 卸载失败"
            fi
            log_warn ${msg}
        elif [ "${insmod_result}" == "PASS" ] && [ "${rmmod_result}" == "PASS" ]; then
            pass_num=$[ ${pass_num}+1 ]
        fi
    done < ${KO_REPORT}

    # 搜出ko文件但没有可测ko文件的也视为fail，后期改为warning
    if [ ${fail_num} -ne 0 ] || [ ${pass_num} -eq 0 ]; then
        return 1
    else
        return 0
    fi
}