#! /bin/bash
################################################################################
#Function: process server packages
#Platform: arm64/amd64/mips64el
#Author: Rongjia Du
#Date: 2022/06/14
#Company: Kylin Software Co.,Ltd.
################################################################################
source ./kylincompatibilitytest/config/config
source ./pub/baselib.sh
source ./pub/log.sh

## 运行服务器应用可执行程序
## 入参：应用所有可执行二进制文件
## 返回值：运行可执行二进制文件结果状态，0-pass，1-fail
#function run_cmd()
#{
#    local cmd_list=$*
#    local fail_num=0
#    local exec_num=0
#    for i in ${cmd_list[@]}
#    do
#        strings $i | grep -- --help >/dev/null 2>&1 # 找出当前应用入口文件
#        if [ $? -eq 0 ];then # 如果应用文件返回结果正确
#            timeout 5 $i --help >/dev/null 2>&1 # 运行当前应用文件--help
#            if [ $? -ne 0 ];then # 如果应用可执行二进制文件--help返回错误
#                fail_num=$[fail_num+1]
#                log_warn "run cmd help result, rpm name: ${PKG_PACKAGENAME}, cmd: $i, state: FAIL"
#            else
#                log_info "run cmd help result, rpm name: ${PKG_PACKAGENAME}, cmd: $i, state: PASS"
#            fi
#            exec_num=$[exec_num+1]
#        fi
#    done
#    # 实际没有执行任何命令
#    if [ ${exec_num} -eq 0 ];then
#        return 2
#    # 以下执行了命令
#    # 失败次数为0，则认为执行成功
#    elif [ ${fail_num} -eq 0 ]; then
#        return 0
#    else
#        return 1
#    fi
#}


# 运行服务器应用可执行程序
# 入参：应用所有可执行二进制文件
# 返回值：运行可执行二进制文件结果状态，0-pass，1-fail
function run_cmd()
{
    local cmd_list=$*
    local fail_num=0
    for i in ${cmd_list[@]}
    do
        strings $i | grep -- --help >/dev/null 2>&1 # 找出当前应用入口文件
        if [ $? -eq 0 ];then # 如果应用文件返回结果正确
            log_info "执行命令$i --help命令"
            timeout 5 $i --help >/dev/null 2>&1 # 运行当前应用文件--help
            local ret=$?
        else
            log_info "无--help参数，直接执行$i命令"
            timeout 5 $i >/dev/null 2>&1 # 运行当前应用文件
            local ret=$?
        fi
        if [ ${ret} -ne 0 ];then # 如果应用可执行二进制文件--help返回错误
            fail_num=$[fail_num+1]
            log_error "run cmd help result, rpm name: ${PKG_PACKAGENAME}, cmd: $i, state: FAIL"
        else
            log_info "run cmd help result, rpm name: ${PKG_PACKAGENAME}, cmd: $i, state: PASS"
        fi
    done

    # 失败次数为0，则认为执行成功
    if [ ${fail_num} -eq 0 ]; then
        return 0
    else
        return 1
    fi
}

# 运行服务器应用service文件
# 入参：应用service文件信息
# 返回值：运行service文件结果状态， 0-pass，1-fail
function run_service()
{
    local service_list=$*
    local num=0
    for i in ${service_list[@]}
    do
        for j in "start" "status" "stop"
        do
            timeout 10 echo ${PASSWD} | sudo -S systemctl ${j} ${i} > /dev/null 2>&1
            if [ $? -ne 0 ];then # 如果服务某一项状态返回码不为0
                num=$[num+1]
                log_error "run service action result, rpm name: ${PKG_PACKAGENAME}, service: $i, action: $j, state: FAIL"
            else
                log_info "run service action result, rpm name: ${PKG_PACKAGENAME}, service: $i, action: $j, state: PASS"
            fi
        done
    done
    if [ ${num} -eq 0 ];then # 应用全部service文件启停状态正常
        return 0
    else
        return 1
    fi
}

# 服务器应用服务启停测试
function start_app_service()
{
    local service_list=($(rpm -ql ${PKG_PACKAGENAME} | grep -E "[^@]\.service$" | awk -F "/" '{print $NF}')) # 获取当前应用所有service格式文件
    log_info "当前服务器的service列表为：${service_list[*]}"
    if [ "${service_list[*]}"x != ""x ];then # 如果当前应用包含service文件
        run_service ${service_list[@]}
        local ret=$?
    else
        local ret=2
    fi
    return ${ret}
}

# 服务器应用命令调用测试
function start_app_cmd()
{
    # 获取当前应用所有可运行程序
    local cmd_list=()
    local bin_list=($(rpm -ql ${PKG_PACKAGENAME} | grep -E '/s?bin/'))
    log_info "当前服务器bin/sbin目录下的文件列表为：${bin_list[*]}"
    for i in ${bin_list[@]}
    do
        file $i | grep -E "\<executable\>|\<ELF\>"  > /dev/null
        if [ "$?" == "0" ];then
            cmd_list=($(echo ${cmd_list[*]} $i))
        fi
    done
    log_info "当前服务器的bin/sbin目录下可执行文件cmd列表为：${cmd_list[*]}"
    if [ "${cmd_list[*]}"x != ""x ];then # 如果当前应用包含cmd命令行
        run_cmd ${cmd_list[@]}
        local ret=$?
    else
        local ret=2
    fi
    return ${ret}
}