#!/bin/bash
#######################################################################################
#Function: setup of applications
#Platform:all
#Author: Fenfen Liu
#Date: 2022/1/20
#Company: Kylin Software Co.,Ltd.
#Description: 应用所有用例所需前置条件
########################################################################################
source ./pub/baselib.sh
source ./config/configuration.conf
source ./pub/log.sh

APP_LIST=${PROJECT_NAME}/config/application.list
BLACK_LIST=${PROJECT_NAME}/config/app.blacklist
log_file=log/project_setup.log

# 将单个包信息写入待测包列表
# 参数：对于本地包传入完整的包路径，对于源里下载的包传入包名
# 返回值：无
function write_pkg_into_list() {
    local pkgs=$1
    echo ${pkgs} >> ${APP_LIST}
}

# 测试列表去除黑名单中应用
# 参数：无
# 返回值：无
function exclude_blacklist() {
    sort ${APP_LIST} | uniq > tmp1.list
    sort ${BLACK_LIST} | uniq > tmp2.list
    comm -23 tmp1.list tmp2.list > ${APP_LIST}
    rm -f tmp1.list tmp2.list
}

# 读取路径下的待测包，写入待测包列表
# 入参：存放测试包的路径
# 返回值：无
function write_pkgs_into_list() {
    local pkgs_path=$1
    if [ -d ${pkgs_path} ]; then
        if [ ${SYSTEM} == desktop ]; then
            local pkgs=$(find ${pkgs_path} -name '*.deb')
        else
            local pkgs=$(find ${pkgs_path} -name '*.rpm')
        fi
        for pkg in ${pkgs}
        do
            write_pkg_into_list ${pkg}
        done

        exclude_blacklist
    else
        log_error "请传入目录" | tee -a ${log_file}
    fi
}

# 复制上传的待测列表
# 入参：待测列表文件
# 返回值：无
function copy_list() {
    local file=$1
    if [ -e ${file} ]; then
        # 解决最后一行没有换行符的文件的读取问题
        cat ${file} | xargs | sed 's/ /\n/g' | while read line
        do
            write_pkg_into_list ${line}
        done

        exclude_blacklist
    else
        log_error "${file}不存在！" | tee -a ${log_file}
    fi
}

# 配置待测列表
# 入参：配置测试列表的参数
# 返回值：0——待测列表修改成功，1——待测列表修改失败
function write_app_list() {
    > ${APP_LIST}
    case $1 in
        --pkg|-pkg)
            shift
            if [ "$1"z != ""z ]; then
                write_pkg_into_list $1
            else
                log_error "请传入包" | tee -a ${log_file}
                return 1
            fi
            ;;
        --dir|-dir)
            shift
            if [ "$1"z != ""z ]; then
                write_pkgs_into_list $1
            else
                log_error "请传入包路径" | tee -a ${log_file}
                return 1
            fi
            ;;
        --list_file|-list_file)
            shift
            if [ "$1"z != ""z ]; then
                copy_list $1
            else
                log_error "请传入包列表" | tee -a ${log_file}
                return 1
            fi
            ;;
    esac

    if [ -z "$(cat ${APP_LIST})" ]; then
        return 1
    else
        return 0
    fi
}

# 检查浏览器应用
function check_browser(){
    echo "#########################浏览器测试前置处理开始#########################" >> "${log_file}"
    log_info "如果测试范围包括浏览器或网页版应用，请手动关闭测试机上所有浏览器进程！！" | tee -a "${log_file}"
    read -p "完成后按任意键继续！"
    log_info "浏览器检查完毕" | tee -a "${log_file}"
    echo "#########################浏览器测试前置处理完成#########################" >> "${log_file}"
}

# 项目级别配置
# 入参：配置参数
# 返回值：0——执行成功，1——执行失败
function main(){
    echo "#########################项目前置处理开始#########################" >> "${log_file}"
    echo ${PASSWD} | sudo -S rm -rf ./kylincompatibilitytest/resource/*
    local pkg_options=""
    while [ -n "$1" ]; do
        case $1 in
            --pkg|-pkg)
                shift
                if [ -z "${pkg_options}" ]; then
                    pkg_options="--pkg $1"
                fi
                ;;
            --dir|-dir)
                shift
                if [ -z "${pkg_options}" ]; then
                    pkg_options="--dir $1"
                fi
                ;;
            --list_file|-list_file)
                shift
                if [ -z "${pkg_options}" ]; then
                    pkg_options="--list_file $1"
                fi
                ;;
        esac
        shift
    done

    if [ "${pkg_options}"z != ""z ]; then
        write_app_list ${pkg_options}
        local ret=$?
        if [ "${ret}" == "1" ]; then
            log_error "项目初始化有误，请查看log" | tee -a "${log_file}"
            echo "#########################项目前置处理失败#########################" >> "${log_file}"
            return 1
        else
            log_info "项目初始化成功" | tee -a "${log_file}"
            echo "#########################项目前置处理完成#########################" >> "${log_file}"
        fi
    else
        log_info "传入参数为空，未执行任何项目初始化操作" | tee -a ${log_file}
        return 1
    fi

#    if [ "${SYSTEM}"x == "desktop"x ]; then
#        check_browser
#    else
#        log_info "环境为服务器，不涉及浏览器测试" | tee -a "${log_file}"
#    fi

    return 0
}

# 主函数部分
args=( $* )
main ${args[@]}
