#!/bin/bash
#######################################################################################
#Function: function of base libarary
#Platform:amd64\arm64\x86
#Author: Fenfen Liu
#Date: 2021/3/24
#Company: Kylin Software Co.,Ltd.
#Description:
########################################################################################

END_TIME=""

# 显示开始标题,入参为项目名称（config/configuration.conf配置的projectname）
function start_title()
{
	echo "
 ##############################################################################
 #                                                                            #
 #       ---- $1 Test start ----                                              #
 #                                                                            #
 #  @Company: Kylin Software Co.,Ltd.                                         #
 #  @Website: http://www.kylinos.cn                                           #
 #  @Version: 1.0                                                             #
 #  @Mail:    liufenfen@kylinos.cn                                            #
 #  @Date:    $(date +%F)                                                     #
 #                                                                            #
 ##############################################################################
   "
}

# 显示结束标题,入参为项目名称（config/configuration.conf配置的projectname）
function stop_title()
{
	echo "
 ##############################################################################
 #                                                                            #
 #                           !!! $1 Test Over !!!                             #
 #                                                                            #
 ##############################################################################
   "
}

# 提示信息
function help(){
	echo "
用法：
    ./run.sh [(-p|--passwd) password] [-run casename|--include (tag|tag1ANDtag2|tag1NOTtag2)|--exclude tag] [项目自定义参数]
	./run.sh                            运行所有用例
	./run.sh --run casename             运行用户指定的用例
	./run.sh --include tag              运行包含tag的用例运行
	./run.sh --include tag1ANDtag2      运行同时包含tag1和tag2的用例
	./run.sh --include tag1NOTtag2      运行包含tag1但不包含tag2的用例
	./run.sh --exclude tag              运行不包含tag的用例
	./run.sh (-p|--passwd) password     配置测试机密码
   "
}

# 时间

# 两种时间格式
# 标准时间格式：YYYY-mm-dd HH:MM:SS
# 文件时间格式：YYYYMMDDHHMMSS

# 获取当前时间
# 前置条件：无
# 使用方法：直接调用
# 返回值：标准时间格式的当前时间
function get_time() {
	echo $(date "+%F %T")
}

# 获取当前时间
# 前置条件：无
#使用方法：直接调用
# 返回值：文件时间格式的当前时间
function get_file_time() {
	echo $(date "+%Y%m%d%H%M%S")
}

# 获取当前秒数
# 前置条件：无
# 使用方法：直接调用
# 返回值：UNIX系统当前时间的秒数
function get_sec() {
	echo $(date "+%s")
}

# 时间转秒，入参：标准时间
# 前置条件：获取当前标准时间
# 使用方法：time_to_sec ${standard_time}
# 返回值：当前时间秒数
function time_to_sec() {
	echo $(date -d "$1" "+%s")
}

# 获取开始时间
# 前置条件：无
# 使用方法：get_start_sec
# 返回值：无，变量start_time会被赋值，可直接调用
function get_start_sec() {
	local start_time=$(get_sec)
    sed -i "s|START_TIME=.*|START_TIME=${start_time}|" "./config/configuration.conf"
    source ./config/configuration.conf
}

# 获取结束时间
# 前置条件：无
# 使用方法：get_end_sec
# 返回值：无，变量end_time会被赋值，可直接调用
function get_end_sec() {
	END_TIME=$(get_sec)
}

# 秒转标准时间，入参：秒
# 前置条件：获取当前秒数
# 使用方法：sec_to_time ${seconds}
# 返回值：标准格式时间
function sec_to_time() {
	echo $(date -d "1970-01-01 UTC $1 seconds" "+%F %T")
}

# 秒转文件时间，入参：秒
# 前置条件：获取当前秒数
# 使用方法：sec_to_filetime ${seconds}
# 返回值：文件格式时间
function sec_to_filetime() {
	echo $(date -d "1970-01-01 UTC $1 seconds" "+%Y%m%d%H%M%S")
}

# 获取时长, 入参（param1：开始时间戳，param2：结束时间戳）
# 前置条件：获取开始时间和结束时间
# 使用方法：get_duration ${start_time} ${end_time}
# 返回结果：两个时间的时间差，格式为HH:MM:SS
function get_duration() {
	local time=$[ $2 - $1 ]
	echo $(date -d "1970-01-01 ${time} seconds" +%T)
}

# 更改一个配置项的值
# 参数1：配置项
# 参数2：对应值
function change_setting() {
    local key=$1
    local value=$2
    sed -i "./config/configuration.conf" -e 's|'"${key}"'=.*|'"${key}"'='"${value}"'|'
    source ./config/configuration.conf
}

# 修改预计执行用例数量
function set_total_case_num() {
    local total_case_num=$1
    change_setting "TOTAL_CASE_NUM" ${total_case_num}
}

# 修改实际执行用例数量
function set_exec_case_num() {
    change_setting "EXEC_CASE_NUM" "$[ ${EXEC_CASE_NUM} + 1 ]"
}

# 执行用例数量清零
function set_zero_case_num() {
    change_setting "TOTAL_CASE_NUM" 0
    change_setting "EXEC_CASE_NUM" 0
}

# 传入时间，将需要用到的与时间戳有关的文件名写入配置文件，入参：开始秒数
# 前置条件：获取开始时间秒数
# 使用方法：write_config_file ${time}
# 结果：将两个日志文件和报告文件路径写入配置文件
function write_config_file() {
    local filetime=$(sec_to_filetime $1)
    local basepath=$(pwd)
    local environment_csv_file="${basepath}/report/environment_${filetime}.csv"
    sed -i "s|ENVIRONMENT_CSV_FILE=.* # 环境信息文件，csv格式|ENVIRONMENT_CSV_FILE=\"${environment_csv_file}\" # 环境信息文件，csv格式|" "./config/configuration.conf"
    local report_csv_file="${basepath}/report/report_${filetime}.csv"
    sed -i "s|REPORT_CSV_FILE=.* # 结果文件路径，csv格式|REPORT_CSV_FILE=\"${report_csv_file}\" # 结果文件路径，csv格式|" "./config/configuration.conf"
#    local case_log_file="${basepath}/log/case_log_${filetime}.log"
#    sed -i "s|CASE_LOG_FILE=.* # 用例运行过程日志，开发者自行输出|CASE_LOG_FILE=${case_log_file} # 用例运行过程日志，开发者自行输出|" "./config/configuration.conf"
    local run_log_file="${basepath}/log/run_log_${filetime}.log"
    sed -i "s|RUN_LOG_FILE=.* # 运行日志文件路径|RUN_LOG_FILE=\"${run_log_file}\" # 运行日志文件路径|" "./config/configuration.conf"
    local project_name=$(echo ${PROJECT_NAME/test/} | tr "[:lower:]" "[:upper:]")
    local report_html_file="${basepath}/report/${project_name}_TEST_REPORT_${filetime}.html"
    sed -i "s|REPORT_HTML_FILE=.* # 报告文件路径|REPORT_HTML_FILE=\"${report_html_file}\" # 报告文件路径|" "./config/configuration.conf"
    local project_path="${basepath}/${PROJECT_NAME}"
    sed -i "s|PROJECT_PATH=.* # 项目路径|PROJECT_PATH=\"${project_path}\" # 项目路径|" "./config/configuration.conf"
    source ./config/configuration.conf
    log_info "[Blazers]修改配置文件完成" | tee -a ${RUN_LOG_FILE}
}

# 修改详细日志路径
# 使用方法：change_log_path ${name}
# 结果：修改CASE_LOG的路径，格式为$TESTSUIT.log
function change_log_path() {
    local testsuit_name=${1##*/}
    if [[ "${testsuit_name}" =~ .(deb|rpm)$ ]]; then
        testsuit_name="${testsuit_name%.*}"
    fi
    local case_log_file=$(pwd)/log/caselog/${testsuit_name}.log
    sed -i "s|CASE_LOG_FILE=.* # 用例运行过程日志，开发者自行输出|CASE_LOG_FILE=\"${case_log_file}\" # 用例运行过程日志，开发者自行输出|" "./config/configuration.conf"
    source ./config/configuration.conf
    log_info "[Blazers]当前用例日志路径为$CASE_LOG_FILE" | tee -a ${RUN_LOG_FILE}
}

# 获取用例日志
# 使用方法：get_case_log ${keyword}
# 返回值：返回带casenum的行之间的日志
function get_case_log() {
    local keyword=$1
    # 找出日志中带有关键词的行号
    local num_list=($(cat ${CASE_LOG_FILE} | grep -n ${keyword} | awk -F ':' '{print $1}'))
    # 返回带有关键词的最大范围日志
    cat ${CASE_LOG_FILE} | sed -n ''${num_list[0]}','${num_list[-1]}'p' > log/case.log
}

# 断言

# 返回值1代表fail，0代表pass

# 判断两个字符串是否相等，入参（param1：预判结果，param2：实际结果）
function assert_equal_string() {
	if [ "$1"x = "$2"x ]; then
		return 0
	else
		return 1
	fi
}

# 判断两个字符串是否不相等，入参（param1：预判结果，param2：实际结果）
function assert_not_equal_string() {
	if [ "$1"x != "$2"x ]; then
		return 0
 	else
		return 1
 	fi
}

# 判断两个数字是否相等，入参（param1：期望值，param2：实际值）
function assert_equal_num() {
	if [ $1 -eq $2 ]; then
		return  0
	else
		return 1
	fi
}

# 判断两个数字是否不相等，入参（param1：期望值，param2：实际值）
function assert_not_equal_num() {
	if [ $1 -ne $2 ]; then
		return 0
	else
		return 1
	fi
}

# 判断文件中是否包含某个关键字，入参（param1：文件路径，param2：字符串）
function assert_string_contain_by_file() {
	local res=$(cat "$1" | grep "$2")
	return $(assert_not_equal_string ${res} "")
}

# 判断终端命令是否成功，需要紧跟在判断的命令后面执行
function assert_command_success() {
	if [ "$?" = "0" ]; then
		return 0
	else
		return 1
	fi
}

# 检查文件状态，如果文件存在且不为空，返回00；否则返回01。入参（param1：字符串）
# 示例： check_file $run_log
function check_file()
{
	if test -s $1; then
		echo "true"
		return 0
	else
		echo "false"
		return 1
	fi
}

# 判断软件是否已在系统上安装,如果已安装，返回true，否则返回false。入参（param1：字符串）
# 示例： check_install "mutt"
function check_install(){
	local ret=0
	# shellcheck disable=SC2068
	for i in $@
	do
		dpkg -l |grep -w "\ $i\ " > /dev/null
		ret=$?
		if [ "${ret}" -ne "0" ];then
			log_info "$i is not installed!" >> $RUN_LOG_FILE
			echo "false"
			return ${ret}
		else
			log_info "$i is already installed!" >> $RUN_LOG_FILE
			echo "true"
			return ${ret}
		fi
	done
}

# 功能：卸载已安装桌面应用
# 入参（param1：字符串（桌面应用包名）；
function deb_uninstall()
{
    local packagename=$1
    log_info "${packagename} 软件卸载"
    echo ${PASSWD} | sudo -S apt-get remove -y ${packagename}
    local ret=$?
    log_info "卸载命令的结果是${ret}"
    local install_status=$(dpkg -l ${packagename} | tail -1 | awk '{printf $1}')
    log_info "应用${packagename}的状态是${install_status}"
    if [ "${ret}" == "0" ] && ([ "${install_status}"x == "rc"x  ] || [ "${install_status}"x == "un"x  ] || [ "${install_status}"x == ""x  ]); then
        log_info "应用${packagename}卸载成功"
        return 0
    else
        log_error "应用${packagename}卸载失败"
        return 1
    fi
}

# 功能：安装桌面应用
# # 入参（param1：字符串（包名或者是deb包全路径）；
function deb_install()
{
    echo $1 | grep ".deb$"
    if [ "$?" = "0" ]; then
        local packagename=$(dpkg -f $1 Package)
    else
        local packagename=$1
    fi
    log_info "${packagename} 开始安装"

    # 开启监控进程
    local cmd="DEBIAN_FRONTEND=noninteractive apt-get install -y -f -m $1"
    (sleep $EXECUTE_T && {
        local case_pid=($(pgrep -f "$cmd"))
        local dpkg_pid=($(ps -ef |grep dpkg | awk '{print $2}'))
        test -n "${case_pid[*]}" && {
            log_error "安装超时"
            log_info "杀死安装主进程 ${case_pid[*]}"
            kill_pid "${case_pid[*]}"
            log_info "杀死dpkg相关进程 ${dpkg_pid[*]}"
            # 杀死dpkg进程
            kill_pid "${dpkg_pid[*]}"
            handle_dpkg_exception ${packagename}
        }
    }) 2>/dev/null &

    # 运行安装
    echo ${PASSWD} | sudo -S $cmd
    local cmd_ret=$?

    # 杀死sleep进程
    sleep_pid=$(pgrep -f "sleep ${EXECUTE_T}")
    if test -n "${sleep_pid}"; then
        kill_pid "${sleep_pid}"
    fi

    # 判断安装状态
    deb_check_install_status ${packagename}
    local check_status_ret=$?
    if [ "${cmd_ret}"x == "0"x -a "${check_status_ret}"x == "0"x ];then
        log_info "应用${packagename} 安装成功"
        return 0
    else
        log_error "应用${packagename} 安装失败"
        local dpkg_pid=($(ps -ef |grep dpkg | awk '{print $2}'))
        log_info "杀死dpkg相关进程 ${dpkg_pid[*]}"
        # 杀死dpkg进程
        kill_pid "${dpkg_pid[*]}"
        handle_dpkg_exception ${packagename} > /dev/null 2>&1
        return 1
    fi
}

# 检查桌面应用是否已安装
# 入参（param1：字符串（桌面应用包名）；
function deb_check_install_status()
{
    local packagename=$1
    local install_status=$(dpkg -l ${packagename} | tail -1 | awk '{print $1}')
    log_info "应用${packagename}的安装状态是${install_status}"
    if [ "${install_status}" = "ii" ]; then
        return 0
    elif [ "${install_status}"x == "rc"x  ] || [ "${install_status}"x == "un"x  ] || [ "${install_status}"x == ""x  ]; then
        return 1
    else
        return 2
    fi
}

# 功能：卸载已安装服务器应用
# # 入参（param1：字符串（服务器应用包名）；
function rpm_uninstall()
{
    local packagename=$1
    log_info "${packagename}软件卸载"
    echo ${PASSWD} | sudo -S yum remove -y ${packagename}
    local ret=$?
    log_info "卸载命令的结果是${ret}"
    # 判断安装状态
    rpm_check_install_status ${packagename}
    local check_status_ret=$?
    if [ "${ret}" == "0" -a "${check_status_ret}" == "1" ]; then
        log_info "应用${packagename}卸载成功"
        return 0
    else
        log_error "应用${packagename}卸载失败"
        return 1
    fi
}

# 功能：安装服务器应用
# # 入参（param1：字符串（包名或者是rpm包全路径）；
function rpm_install()
{
    echo $1 | grep ".rpm$"
    if [ "$?" = "0" ]; then
        local packagename=$(rpm -qi $1 | grep ^Name | awk -F : '{print $2}' | sed -e 's/ //g')
    else
        local packagename=$1
    fi
    log_info "${packagename} 开始安装"

    # 开启监控进程
    local cmd="yum install -y $1"
    (sleep $EXECUTE_T && {
        case_pid=($(pgrep -f "$cmd"))
        test -n "${case_pid[*]}" && {
            log_error "安装超时"
            log_info "杀死安装主进程 ${case_pid[*]}"
            kill_pid "${case_pid[*]}"
        }
    }) 2>/dev/null &

    # 运行安装
    echo ${PASSWD} | sudo -S $cmd
    local cmd_ret=$?

    # 杀死sleep进程
    sleep_pid=$(pgrep -f "sleep ${EXECUTE_T}")
    if test -n "${sleep_pid}";then
        kill_pid "${sleep_pid}"
    fi

    # 判断安装状态
    rpm_check_install_status ${packagename}
    local check_status_ret=$?
    if [ "${cmd_ret}"x == "0"x -a "${check_status_ret}"x == "0"x ];then
        log_info "应用${packagename} 安装成功"
        return 0
    else
        log_error "应用${packagename} 安装失败"
        return 1
    fi
}

# 检查服务器应用是否已安装
# 入参（param1:字符串（服务器应用包名）;
function rpm_check_install_status()
{
    local packagename=$1
    yum list ${packagename} | grep -E "已安装的软件包|Installed Packages" >/dev/null
    if [ $? -eq 0 ];then
        log_info "应用${packagename}的安装状态是已安装的软件包"
        return 0
    else
        return 1
    fi
}

# 杀灭新增进程
function kill_pid()
{
    local pid_array=($*)
    # shellcheck disable=SC2068
    for pid in ${pid_array[@]}
    do
        log_info "杀死进程 ${pid}"
        echo ${PASSWD} | sudo -S kill -9 $pid > /dev/null 2>&1 &
    done
}

# 处理dpkg锁异常
# 入参（param1：字符串（应用包名）；
function handle_dpkg_exception()
{
    local packagename=$1
    echo ${PASSWD} | sudo -S dpkg --configure -a
    echo ${PASSWD} | sudo -S dpkg --remove --force-remove-reinstreq ${packagename}
}

# 完成本机文件与远程服务器的文件相互拷贝功能。
# 入参（param1：字符串（源文件路径）；param2：字符串（目标路径）；param3：字符串（远程密码）；param4：字符串（端口号，可不做输入，默认为22端口））
# 示例：将本机文件拷贝至远程服务器： sshscp "/home/kylin/1.txt" "kylin@172.17.31.90:/home/kylin/" "qwer1234"
# 示例：将远程服务器文件拷贝至本机：sshscp  "kylin@172.17.31.90:/home/kylin/1.txt" "/home/kylin/"  "qwer1234"
# 注：脚本中默认scp -P $connect_port 端口为默认22，如果端口有变化，请将端口号做为第四个参数传入。
function sshscp() {
	local ret=$(check_install "expect")
	local source_file=$1
	local dest=$2
	local connect_passwd=$3
	local connect_port=${4-22}
	if [ $ret == "false" ]; then
		install_dep "expect"
	fi

	test "$source_file"x = ""x && log ERROR  "文件不存在，请确认待传输文件是否存在" >> $RUN_LOG_FILE

	test "$dest"x = ""x && log ERROR  "目标目录不存在，请确认目的目录是否存在" >> $RUN_LOG_FILE

	test "$connect_passwd"x = ""x &&  log ERROR "远程连接登录密码为空，请确认密码是否正确" >> $RUN_LOG_FILE

	test "$connect_port"x = "22"x && log WARN  "远程连接端口为默认端口22" >> $RUN_LOG_FILE

	expect <<-EOF
        set timeout -1
        spawn scp -P $connect_port -r $source_file $dest
        expect {
            "Are you sure you want to continue connecting*"
            {
                send "yes\r"
                expect "\[P|p]assword:"
                send "${connect_passwd}\r"
            }
            -re "\[P|p]assword:"
            {
                send "${connect_passwd}\r"
            }
            timeout
            {
                send_user "connection to remote timed out: \$expect_out(buffer)\n"
                exit 101
            }

        }
        expect eof
EOF
	exit $?
}

# 检查网络连接，连接IP或者域名;如果连接失败，尝试启动网络服务；默认超过10次仍不能连接，报连接失败错误。入参（param1：字符串）
# 示例： check_network 172.17.111.22，
function check_network()
{
	local remote_ip=$1
	local packet=3 # ping网络时发送包的个数
	local count=10 # 网络启动重试次数
	local i=1
	test "$remote_ip"x = ""x &&  log_error  "未输入IP，IP为空!"
	log_info "正在检查网络环境"
	while [ $i -le $count ]
	do
		local result=$(ping -c $packet ${remote_ip} | grep  "$packet received" | grep "0% packet loss")
		if  [ "${result}"x != ""x ]; then
			log_info "connection to $remote_ip successfully."
			break
		else
			log_error "connection to $remote_ip failed. Try to restart networking service"
			echo ${PASSWD} | sudo -S systemctl restart networking
			let i++
			sleep 2
		fi
	done

	local res=0
	if [ $i -gt $count ] ; then
		log_error "connection to $remote_ip failed. please check the network"
		res=1
	fi
	log_info "网络环境检查完毕"
	return ${res}
}

# 删除过期文件，configuration.conf的文件中配置保留文件个数
# 入参param1：字符串（待删除文件所在目录）
# 入参para2：保留文件个数
# 示例 remove_expired_files /home/kylin/xframework/report 2
function remove_expired_files()
{
	local file_reserve_num=$2
	local file_dir=$1
	local file_num=$(ls -l $file_dir | grep ^- | wc -l)
	# empty or non-positive  keep all logs
	if [ x"$file_reserve_num" == x"" ];then
		log_debug "保留文件个数为空或者未设置" >> $RUN_LOG_FILE
		return
	fi
	if [ $file_reserve_num -lt 0 ]; then
		log_debug "保留文件个数设置错误，不可小于0" >> $RUN_LOG_FILE
		return
	fi

	# 删除过期文件
	while (($file_num > $file_reserve_num))
	do
		old_file=$(ls -rtl $file_dir | grep ^- | head -1 | awk '{print $9}')
		log_debug "删除过期文件$old_file" >> $RUN_LOG_FILE
		rm -rf "$file_dir/$old_file"
		let "file_num--"
	done
}

# 将每次测试的结果打包存储,压缩包名为测试结束时间
# 参数：测试结束时间
function package_result(){
	local start_time=$(sec_to_filetime $1)
	log_info "[Blazers]正在打包结果..." | tee -a ${RUN_LOG_FILE}
	tar -zcvf ./pack/${start_time}.tar.gz ./log ./report > /dev/null
	if [ -f ./pack/${start_time}.tar.gz ]; then
		log_info "[Blazers]结果打包完毕，请查看pack文件夹下的${start_time}.tar.gz" | tee -a ${RUN_LOG_FILE}
	else
		log_info "[Blazers]结果打包失败" | tee -a ${RUN_LOG_FILE}
	fi
}

# 创建目录
# 参数：目录路径
function create_directory() {
	mkdir $1
}

# 判断平台类型是桌面还是服务器
# 结果写入配置文件的SYSTEM
function get_system() {
    local os_name="$(cat /etc/os-release | grep "^NAME" | awk '{print $4}' | sed -e 's/\"//')"
    if [ "${os_name}"x == "Server"x ]; then
        sed -i "s|SYSTEM=.*|SYSTEM=\"server\"|" "./config/configuration.conf"
    else
        sed -i "s|SYSTEM=.*|SYSTEM=\"desktop\"|" "./config/configuration.conf"
    fi
    source ./config/configuration.conf
}

# 传入时间，将测试机密码写入配置文件
# 入参：测试机密码
# 使用方法：write_passwd ${passwd}
# 返回值：无
function write_passwd() {
    local passwd=$1
    sed -i "s|PASSWD=.*|PASSWD=\"${passwd}\"|" "./config/configuration.conf"
    source ./config/configuration.conf
    log_info "[Blazers]测试机密码配置完成" | tee -a ${RUN_LOG_FILE}
}

# 输出提示语
function print_kysec_setting_hints(){
    log_info "请您到操作系统设置菜单栏中的设置->安全中心->应用控制与保护菜单下手动关闭相关配置项" | tee -a "${RUN_LOG_FILE}"
    read -p "完成后按任意键继续！"
    log_info "安全中心配置手动关闭完成" | tee -a "${RUN_LOG_FILE}"
}

# 检查安全中心设置
function check_kylinsec(){
    log_info "正在检查安全中心设置" | tee -a "${RUN_LOG_FILE}"
    if [ "${SYSTEM}"x == "desktop"x ]; then
        log_info "系统为桌面系统" | tee -a "${RUN_LOG_FILE}"
        deb_check_install_status "kylin-signtool" >> /dev/null
        local res1=$?
        dpkg -L kylin-signtool | grep setsignstatus >> /dev/null
        local res2=$?
        if [ ${res1} -eq 0 -a ${res2} -eq 0 ]; then
            log_info "系统支持自动设置签名检查功能" >> "${RUN_LOG_FILE}"
            
            echo ${PASSWD} | sudo -S setsignstatus off > /dev/null
            local res1=$?
            # getsignstatus | grep "kylin sign status : off"
            # local res2=$?
            
            setstatus -f exectl off > /dev/null
            local res3=$?
            getstatus | grep "exec control[ ]*: off" > /dev/null
            local res4=$?
                
            setstatus -f ppro off > /dev/null
            local res5=$?
            getstatus | grep "process protect[ ]*: off" > /dev/null
            local res6=$?

            setstatus -f fpro off > /dev/null
            local res7=$?
            getstatus | grep "file protect[ ]*: off" > /dev/null
            local res8=$?

            setstatus -f kmod off > /dev/null
            local res9=$?
            getstatus | grep "kmod protect[ ]*: off" > /dev/null
            local res10=$?

            if [ ${res1} -eq 0 -a ${res4} -eq 0 -a ${res6} -eq 0 -a ${res8} -eq 0 -a ${res10} -eq 0 ]; then
                log_info "已成功自动化关闭安全中心->应用控制与保护界面安全相关的三项配置，继续测试！" | tee -a "${RUN_LOG_FILE}"
            else
                log_info "kylin-signtool签名配置关闭命令结果为"${res1}"" >> "${RUN_LOG_FILE}"
                log_info "执行控制关闭命令结果为"${res3}"，状态查询结果为"${res4}"" >> "${RUN_LOG_FILE}"
                log_info "进程保护关闭命令结果为"${res5}"，状态查询结果为"${res6}"" >> "${RUN_LOG_FILE}"
                log_info "文件保护关闭命令结果为"${res7}"，状态查询结果为"${res8}"" >> "${RUN_LOG_FILE}"
                log_info "内核模块保护关闭命令结果为"${res9}"，状态查询结果为"${res10}"" >> "${RUN_LOG_FILE}"
                print_kysec_setting_hints
	        fi
        else
            log_info "系统不支持自动关闭签名检查功能" >> "${RUN_LOG_FILE}"
            print_kysec_setting_hints
        fi
    else
        log_info "系统为服务器系统，无此设置项" | tee -a "${RUN_LOG_FILE}"
    fi
    log_info "安全中心设置检查完毕" | tee -a "${RUN_LOG_FILE}"
}

# 自动备份
function backup() {
    local tool_path=$(pwd)
    local des_path=${tool_path%/*}

    if [ -d ${des_path}/sysrestore/ ]; then
        # 如果已经存在工具，则检查是否完成备份
        cd ${des_path}/sysrestore
        if [ -e ./run_recover.sh ]; then
            local backup_path=$(cat ./run_recover.sh | awk '{print $2}')
            if [ -d ${backup_path} ]; then
                log_info "系统已完成备份" | tee -a ${RUN_LOG_FILE}
                cd - > /dev/null
                return 0
            fi
        fi
    else
        # 如果没有工具则复制工具到指定路径
        cp -r ./tools/sysrestore/ ${des_path}
        cd ${des_path}/sysrestore
        echo ${PASSWD} | sudo -S chmod -R a+x ./*
    fi

    echo ${PASSWD} | sudo -S ./system_backup.sh
    if [ $? -eq 0 ]; then
        log_info "系统备份成功" | tee -a ${RUN_LOG_FILE}
        cd - > /dev/null
    else
        log_error "系统备份失败，请手动备份成功后再执行工具" | tee -a ${RUN_LOG_FILE}
        exit 1
    fi
}

# 设置工具开机自启动
function set_autostart() {
# 在/etc/profile.d下创建blazers_start.sh 脚本，在用户登录时脚本会被调用
	local scripts="/etc/profile.d/blazers_start.sh"
	if [ -e ${scripts} ]; then
		log_info "系统下已存在工具自启动脚本 ${scripts}" | tee -a ${RUN_LOG_FILE}
	else
		log_info "系统下无工具自启动脚本" | tee -a ${RUN_LOG_FILE}
		echo ${PASSWD} | sudo -S touch ${scripts}
		if [ -e ${scripts} ]; then
			log_info "成功创建工具自启动脚本${scripts}" | tee -a ${RUN_LOG_FILE}
		else
			log_error "自动创建工具启动脚本失败，请手动创建后运行工具" | tee -a ${RUN_LOG_FILE}
			exit 1
		fi
	fi

	cat ${scripts} | grep "cd .* && ./run.sh" > /dev/null
	if [ $? -ne 0 ]; then
		echo ${PASSWD} | sudo -S chmod a+w ${scripts}
		local path=$(pwd)
		cat > ${scripts} << EOF
#!/bin/bash

process=\$(ps -ef | grep "run.sh" | grep -v "grep")
if [ "\${process}"x == x ]; then
	mate-terminal -e 'bash -c "cd ${path} && ./run.sh; exec bash"'&
fi
EOF
		echo ${PASSWD} | sudo -S chmod a+x ${scripts}
		log_info "工具启动命令已写入${scripts}，自启动配置完成" | tee -a ${RUN_LOG_FILE}
	else
		log_info "当前系统已完成自启动工具配置，无需再次配置" | tee -a ${RUN_LOG_FILE}
	fi	
}

# 将命令移出/etc/profile文件，不影响正常开机
function remove_autostart() {
	local scripts="/etc/profile.d/blazers_start.sh"
	echo ${PASSWD} | sudo -S rm -f ${scripts}
	if [ -e ${scripts} ]; then
		log_error "工具自启动脚本${scirpts}移除失败" | tee -a ${RUN_LOG_FILE}
	else
		log_info "工具自启动脚本已移除" | tee -a ${RUN_LOG_FILE}
	fi
}

# 设置用户自动登录
function set_autologin() {
    # 在 /etc/lightdm/lightdm.conf 中加入相关信息
    local conf="/etc/lightdm/lightdm.conf"
    if [ -e ${conf} ]; then
        cat ${conf} | grep "^autologin-user=kylin" > /dev/null
        if [ $? -ne 0 ]; then
            cat ${conf} | grep "^autologin-user=" > /dev/null
            if [ $? -ne 0 ]; then
                echo ${PASSWD} | sudo -S sed -i ${conf} -e '/[Seat.*]/s/]$/]\nautologin-user=kylin/'
            else
                echo ${PASSWD} | sudo -S sed -i ${conf} -e 's/^autologin-user=.*/autologin-user=kylin/'
            fi
            log_info "用户自动登录配置完成" | tee -a ${RUN_LOG_FILE}
        else
            log_info "用户自动登录配置已完成" | tee -a ${RUN_LOG_FILE}
        fi
    else
        log_info "本机未发现${conf}文件，自动创建" | tee -a ${RUN_LOG_FILE}
        echo ${PASSWD} | sudo -S touch ${conf}
        if [ -e ${conf} ]; then
            log_info "${conf}文件创建完成" | tee -a ${RUN_LOG_FILE}
            echo ${PASSWD} | sudo -S chmod a+w ${conf}
            echo ${PASSWD} | sudo -S echo "[SeatDefaults]" >> ${conf}
            echo ${PASSWD} | sudo -S echo "autologin-user=kylin" >> ${conf}
            cat ${conf} | grep "^autologin-user=kylin" > /dev/null
            if [ $? -eq 0 ]; then
                log_info "用户自动登录配置完成" | tee -a ${RUN_LOG_FILE}
            else
                log_error "用户自动登录配置失败，请用户手动配置用户自动登录后执行工具" | tee -a ${RUN_LOG_FILE}
                exit 2
            fi
        else
            log_error "创建${conf}文件失败，请用户手动配置用户自动登录后执行工具" | tee -a ${RUN_LOG_FILE}
            exit 1
        fi
    fi
}

# 保存测试机密码
function set_password()
{
    local args=( $* )
    while [ -n "$1" ]; do
        case $1 in
        -p|--passwd|-passwd)
            shift
            local passwd=$1
            if [ "${passwd}"z == ""z ]; then
                log_error "请传入测试机密码后运行工具" | tee -a ${RUN_LOG_FILE}
                exit
            fi
            write_passwd ${passwd}
        esac
        shift
    done
}
