#!/bin/bash
#######################################################################################
#Function: function of case schedule
#Platform:amd64\arm64\x86
#Author: Lili Sun
#Date: 2021/04/08
#Company: Kylin Software Co.,Ltd.
#Description:
# 1) If you wanted to run cases you could explicitly include them, or exclude them,
#    you can use these functions with tags
# 2) other functions about get all cases path, case suite,case name ,case tags
# 3) If you wanted to know how to define tags ,please see TestDemo/testcase05.sh
########################################################################################
#source  /home/kylin/PycharmProjects/xframework/config/configuration.conf
source ./config/configuration.conf
source ./pub/log.sh
source ./pub/baselib.sh
source ./pub/report.sh
source ./control/controllib.sh

CASE_TAGS=""
CASE_SUIT=""
CASE_NUM=""
CASE_NAME=""
CASE_DESCRIPTION=""
OLD_SUIT=""

PASS_NUM=0
FAIL_NUM=0
WARN_NUM=0
SKIP_NUM=0
#TOTAL_PASS_NUM=0
#TOTAL_FAIL_NUM=0
CASEPOOL_PASS_NUM=0
CASEPOOL_FAIL_NUM=0
CASE_REPORT=""
APP_SETUP=-1 # 0代表存在应用测试初始化脚本，1代表不存在，-1代表未搜索
APP_SETUP_SUCCESS=1 # 0代表真，1代表假

CASE_INFO_FILE_PATH=${PROJECT_NAME}/config/caseinfo.csv

#根据标签获取待执行的用例,支持两个标签同时满足的情况
#入参：(param1:smoke/ p1 等tag)
#出参：返回值为一维数组，如："common-base-01.sh","desktop,base,p0","软件包文件检查","[common-base-01]：确认软件包为deb包且包中不含其他deb包","","","casepool/desktop"
#调用方式：case_schedule_with_tags smoke #获取tags标签含有smoke的用例
function get_case_list_contain_aANDb(){
	local mytags1=$1
	local mytags2=$2
#	local files=($(find ./$PROJECT_NAME/  -name "*.sh" | xargs grep "Tags" | grep "$mytags1" | grep "$mytags2" | grep -v "#" | grep -v "skip" | sort))
    local files=($(cat ${CASE_INFO_FILE_PATH} | grep "$mytags1" | grep "$mytags2" | grep -v "#" | grep -v "skip" | sort))
	echo ${files[@]}
}

#根据标签获取待执行的用例,支持两个标签满足A但不满足B
#入参：(param1:smoke/ p1 等tag)
#出参：返回值为一维数组，如："common-base-01.sh","desktop,base,p0","软件包文件检查","[common-base-01]：确认软件包为deb包且包中不含其他deb包","","","casepool/desktop"
#调用方式：case_schedule_without_tags smoke #获取tags标签不含smoke的用例
function get_case_list_contain_aNOTb(){
	local mytags1=$1
	local mytags2=$2
#	local files=($(find ./$PROJECT_NAME/  -name "*.sh" | xargs grep "Tags" | grep "$mytags1" |  grep -v "$mytags2" | grep -v "#" | grep -v "skip" | sort))
    local files=($(cat ${CASE_INFO_FILE_PATH} | grep "$mytags1" | grep -v "$mytags2" | grep -v "#" | grep -v "skip" | sort))
	echo ${files[@]}
}

#根据标签获取待执行的用例,支持标签不满足C
#入参：(param1:smoke/ p1 等tag)
#出参：返回值为一维数组，如："common-base-01.sh","desktop,base,p0","软件包文件检查","[common-base-01]：确认软件包为deb包且包中不含其他deb包","","","casepool/desktop"
#调用方式：case_schedule_without_tags smoke #获取tags标签不含smoke的用例
function get_case_list_contain_NOTb(){
	local mytags=$1
#	local files=($(find ./$PROJECT_NAME/  -name "*.sh" | xargs grep "Tags" | grep -v "$mytags" | grep -v "#" | grep -v "skip" | sort))
    local files=($(cat ${CASE_INFO_FILE_PATH} | grep -v "$mytags" | grep -v "#" | grep -v "skip" | sort))
	echo ${files[@]}
}

#支持根据用例名称获取对应测试脚本，调用方式为：case_schedule_with_testcase testcase05.sh
function get_case_list_contain_testcase(){
	local case_script=$1
#	local files=($(find ./$PROJECT_NAME/  -name "*.sh" | xargs grep "Tags" | grep -v "#" | grep "${case_script}" | grep -v "skip" | sort) )
    local files=($(cat ${CASE_INFO_FILE_PATH} | grep "${case_script}" | grep -v "skip" | grep -v "#" | sort))
	echo ${files[@]}
}

#获取所有的测试用例
function get_case_list_contain_all(){
	#获取待执行case列表，包括脚本位置，以及脚本标签，返回格式为数组
#	local files=($(find ./$PROJECT_NAME/"casepool"  -name "*.sh" | sort | xargs grep "Tags"| grep -v "skip"| grep -v "#" | sort))
	local files=($(cat ${CASE_INFO_FILE_PATH} | grep -v "skip" | grep -v "#" | sort))
	echo ${files[@]}
}

# 获取指定平台的测试用例
# 入参：（param1:筛选后文件列表）
# 出参：返回为一维数组，如："common-base-01.sh","desktop,base,p0","软件包文件检查","[common-base-01]：确认软件包为deb包且包中不含其他deb包","","","casepool/desktop"
# 调用方式：get_case_list_contain_system ${case_list[*]}
function get_case_list_contain_system() {
    local file_list=$*
    local files=($(echo ${file_list[*]} | sed -e "s/\ /\n/g"  | grep ${SYSTEM} | sort))
    echo ${files[@]}
}

# 执行包含标签的用例
# 入参：标签，如"smokeANDp0"
# 出参：类型为：数组；"common-base-01.sh","desktop,base,p0","软件包文件检查","[common-base-01]：确认软件包为deb包且包中不含其他deb包","","","casepool/desktop"
function get_case_include(){
	# 支持三种模式
	# 1. 单个tag，执行包含此标签的用例
	# 2. tag1ANDtag2，执行同时包含tag1和tag2的用例
	# 3. tag1NOTtag2，执行包含tag1但不包含tag2的用例
	local case_list=""
	local tag1=""
	local tag2=""
	local factor_and=$(echo $1 | grep .*AND.*)
	local factor_not=$(echo $1 | grep .*NOT.*)

	if [ -n "${factor_not}" ]; then #判断factor_not不为空
		# tag1NOTtag2
		tag1=${1%NOT*} # 去掉第一个NOT和它右边的字符
		tag2=${1##*NOT} # 去掉最后一个NOT和它左边的字符
		case_list=($(get_case_list_contain_aNOTb $tag1 $tag2))
	else
		if [ -n "${factor_and}" ]; then
			# tag1ANDtag2
			tag1=${1%AND*} # 去掉第一个AND和它右边的字符
			tag2=${1##*AND} # 去掉最后一个AND和它左边的字符
		else
			tag1=$1
		fi
		case_list=($(get_case_list_contain_aANDb $tag1 $tag2))
	fi

	if [ ${#case_list[*]} == 0 ]; then
		log_error "用例为0"
		exit
	fi

	echo ${case_list[*]}
}

# 执行不包含标签的用例
# 入参：标签，如"smoke"
# 出参：类型为：数组；"common-base-01.sh","desktop,base,p0","软件包文件检查","[common-base-01]：确认软件包为deb包且包中不含其他deb包","","","casepool/desktop"
function get_case_exclude() {
	if  [ -z "$1" ]; then
		log_error "参数不能为空"
		exit
	fi

	local case_list=($(get_case_list_contain_NOTb $1))

	if [ ${#case_list[*]} == 0 ]; then
		log_error "用例为0"
		exit
	else
		echo ${case_list[*]}
	fi
}

# 获取用例
# 返回caseinfo.csv中筛选所得的行数据
# "common-base-01.sh","desktop,base,p0","软件包文件检查","[common-base-01]：确认软件包为deb包且包中不含其他deb包","","","casepool/desktop"
function get_case() {
	# 主函数部分
	local args=( $* )
	local res=""
	case "${args[0]}" in
		"")
			res=($(get_case_list_contain_all))
		;;
		--run)
			unset args[0]
			res=($(get_case_list_contain_testcase "${args[1]}"))
		;;
		--include)
			res=($(get_case_include "${args[1]}"))
		;;
		--exclude)
			res=($(get_case_exclude "${args[1]}"))
		;;
		*)
			exit
	esac
	echo "${res[@]}"
}

########################################################################################################################

#获取脚本Suit的名称,入参（param1：脚本路径,如"common-base-01.sh","desktop,base,p0","软件包文件检查","[common-base-01]：确认软件包为deb包且包中不含其他deb包","","","casepool/desktop"
function get_suit_name_from_case_info(){
    local case_info=$1
	local tem1=$(echo ${case_info} | awk -F '","' '{print $NF}' | awk -F '/' '{print $NF}' | sed 's/"//')
	if [ "$tem1" = "casepool" ]; then
		suit_name=""
	else
		suit_name=${tem1^} # 将字符串首字母变成大写
	fi
	echo ${suit_name}
}

# 获取关键词对应列
function get_index_of_key(){
    local key=$1
    local case_num=1
    local case_tags=2
    local case_name=3
    local case_description=4
    local case_effect=5
    local case_fix=6
    local case_path=7

    eval echo \$${key}
}

# 获取用例信息中指定的列
# 入参（param1：脚本路径,如 "common-base-01.sh","desktop,base,p0","软件包文件检查","[common-base-01]：确认软件包为deb包且包中不含其他deb包","","","casepool/desktop"
# 入参（param2：关键词
# 返回值：指定列的值
function get_value_from_case_info_by_key(){
    local case_info=$1
    local key=$2

    local key_index=$(get_index_of_key ${key})
    local value=$(echo ${case_info} | awk -F '","' '{print $'${key_index}'}' | sed 's/"//')
    echo ${value}
}

# 从用例信息中获取脚本路径
# 入参：脚本信息，如 "common-base-01.sh","desktop,base,p0","软件包文件检查","[common-base-01]：确认软件包为deb包且包中不含其他deb包","","","casepool/desktop"
# 返回值：脚本的相对路径
function get_case_path_from_case_info()
{
    local case_info=$1
    local case_num=$(get_value_from_case_info_by_key ${case_info} "case_num")
    local case_path=$(get_value_from_case_info_by_key ${case_info} "case_path")
    case_path="./${PROJECT_NAME}/${case_path}/${case_num}.sh"
    echo ${case_path}
}

#执行测试用例，需要传入脚本路径，脚本所需的参数
function exec_script(){
    local script_path=$1
    local casepool_para=$2
    local script_para=$3
    local test_star_time=$(get_time)
    local case_start_time=0
    local case_ret=""
    local case_end_time=""
    local case_duration=""
    local case_info=""
    local case_log=""

    if [ "${CASE_SUIT}" != "" ]; then
        if [ "${OLD_SUIT}" != "${CASE_SUIT}" ];then #通过跟前一条case suit的比较，决定是否需要打印本次suit
            log_info "[Blazers]Test Suit: ${CASE_SUIT}=============================================================================" | tee -a ${RUN_LOG_FILE}
            #create_report_with_csv "suite" "${CASE_REPORT}" "$CASE_SUIT"
        fi
    fi
    log_info "[Blazers]  ||[RUN  SAT]Test Case: ${CASE_NUM}=================================Tags: [${CASE_TAGS}]==========" | tee -a ${RUN_LOG_FILE}

    case_start_time=$(get_sec)
    #bash ${script_path} ${casepool_para} ${script_para} #执行case
    ${script_path} ${casepool_para} ${script_para}
    case_ret=$?

    ### 获取错误日志
    get_case_log ${CASE_NUM}
    if [ $case_ret -eq 0 ]; then #判断结果
        CASE_RESULT="通过"
        PASS_NUM=$(( $PASS_NUM+1 ))
    elif [ $case_ret -eq 1 ]; then
        CASE_RESULT="不通过"
        FAIL_NUM=$(( $FAIL_NUM+1 ))
        case_info=$(cat log/case.log | sed 's/$/\\n/g' | sed 's/\"/\"\"/g')
    elif [ ${case_ret} -eq 2 ]; then
        CASE_RESULT="不涉及"
        SKIP_NUM=$(( $SKIP_NUM+1 ))
        case_info="不涉及此项测试，不做测试"
    elif [ ${case_ret} -eq 3 ]; then
        CASE_RESULT="警告"
        WARN_NUM=$(( $WARN_NUM+1 ))
        PASS_NUM=$(( $PASS_NUM+1 ))
        case_info=$(cat log/case.log | grep '\[WARN\]' | sed 's/$/\\n/g' | sed 's/\"/\"\"/g')
    else
        CASE_RESULT="未知错误"
        FAIL_NUM=$(( $FAIL_NUM+1 ))
        case_info=$(cat log/case.log | sed 's/$/\\n/g' | sed 's/\"/\"\"/g')
    fi
    rm -f log/case.log
    # sleep 2
    case_end_time=$(get_sec)
    case_duration=$(get_duration $case_start_time $case_end_time)

    create_report_with_csv "case" "${CASE_REPORT}" "${CASE_SUIT}" "${CASE_NUM}" "${CASE_NAME}" "${CASE_DESCRIPTION}" "${CASE_RESULT}" "${case_duration}" "${case_info}" "" ""
    log_info "[Blazers]  ||"
    log_info "[Blazers]  ||[RUN  END]Test Case: $CASE_NUM=================================$CASE_RESULT========================" | tee -a ${RUN_LOG_FILE}

    # 实际执行用例数增加
    set_exec_case_num
    OLD_SUIT=$CASE_SUIT
}

# 执行测试用例池，需要传入用例列表和脚本所需参数
# para1: 用例列表
# para2: 应用名称 或 第n次执行casepool
function exec_casepool() {
    local file_paths=$1
    local casepool_para=$2
    local current_runtime=0
    local case_path=""
    local case_num=0
    local app_setup_path=./${PROJECT_NAME}/setup/app_setup.sh

    log_info "[Blazers]Test Runtime: ${casepool_para} =============================================================================" | tee -a ${RUN_LOG_FILE}

    # 修改用例日志路径
    change_log_path ${casepool_para}
    rm -f ${CASE_LOG_FILE}

    # 搜索应用初始化脚本，脚本是否存在只需判断一次
    if [ ${APP_SETUP} -eq -1 ]; then
        log_info "[Blazers]搜索应用测试初始化脚本" | tee -a ${RUN_LOG_FILE}
        if [ -f ${app_setup_path} ]; then
            log_info "[Blazers]找到应用测试初始化脚本" | tee -a ${RUN_LOG_FILE}
            APP_SETUP=0
        else
            log_info "[Blazers]未找到应用测试初始化脚本" | tee -a ${RUN_LOG_FILE}
            APP_SETUP=1
            APP_SETUP_SUCCESS=0
        fi
    fi

    if [ ${APP_SETUP} -eq 0 ]; then
        # 执行项目初始化脚本
        log_info "[Blazers]执行应用测试初始化脚本" | tee -a ${RUN_LOG_FILE}
        #bash ${app_setup_path} ${casepool_para}
        ${app_setup_path} ${casepool_para}
        if [ $? -eq 0 ]; then
            log_info "[Blazers]应用测试初始化成功" | tee -a ${RUN_LOG_FILE}
            APP_SETUP_SUCCESS=0
        else
            log_info "[Blazers]应用测试初始化失败" | tee -a ${RUN_LOG_FILE}
            log_info "[Blazers]退出执行" | tee -a ${RUN_LOG_FILE}
            APP_SETUP_SUCCESS=1
        fi
    fi

    local app_test_start_time=$(get_sec)
    if [ ${APP_SETUP_SUCCESS} -eq 0 ]; then
        local results=""
        for file in ${file_paths[@]}
        do
            case_path=$(get_case_path_from_case_info ${file})
            CASE_TAGS=$(get_value_from_case_info_by_key ${file} "case_tags")
            CASE_SUIT=$(get_suit_name_from_case_info ${file})
            CASE_NUM=$(get_value_from_case_info_by_key ${file} "case_num")
            CASE_NAME=$(get_value_from_case_info_by_key ${file} "case_name")
            CASE_DESCRIPTION=$(get_value_from_case_info_by_key ${file} "case_description")
            CASE_RESULT=""

            #取出控制器中脚本的循环设置
            current_runtime=$(get_para_from_control "$CASE_NUM")
            if [ "${current_runtime}" = "" ]; then
                current_runtime=1
            fi
            #根据设置脚本运行次数的不同，分别处理
            if echo ${current_runtime} | grep -q '[^0-9]'; then
                #如果设置脚本运行次数为非固定的，即文件；case name用文件中的字段代替；循环次数=文件行数
                log_info "[Blazers]从控制器读取执行次数:CASE_NUM:[$CASE_NUM];Run Time:${current_runtime} 文件行数" | tee -a ${RUN_LOG_FILE}

                for line in $(cat ${current_runtime})
                do
                    CASE_NUM=$line
                    exec_script "${case_path}" "${casepool_para}" "${line}"
                done
            else
                #如果设置脚本运行次数为固定的；循环次数=设定的数值
                log_info "[Blazers]从控制器读取执行次数:CASE_NUM:[$CASE_NUM];Run Time:${current_runtime} 次" | tee -a ${RUN_LOG_FILE}
                for((k=1;k<=${current_runtime};k++));do
                    exec_script "${case_path}" "${casepool_para}" "$k"
                done;
            fi
            if [ "${results}"x == ""x ]; then
                results='"'${CASE_RESULT}'"'
            else
                results=${results}',"'${CASE_RESULT}'"'
            fi
        done

        ###1.总结小报告--合计
        case_num=$(( ${PASS_NUM}+${FAIL_NUM}+${SKIP_NUM} ))
        #create_report_with_csv "hint" "${CASE_REPORT}" "结果一栏中：NA代表用例不满足执行条件，未执行；WARNING代表检查项存在兼容性问题，需人工排查验证；Unknown代表用例执行过程有未知错误，需人工排查"
        #create_report_with_csv "result" "${CASE_REPORT}" "共执行用例 ${case_num} 条， 其中通过 ${PASS_NUM} 条（warning ${WARN_NUM}条，skip ${SKIP_NUM}条）， 失败 ${FAIL_NUM} 条"
        echo "########## 应用测试报告路径为 ${CASE_REPORT} ##########"
    fi
    local app_test_end_time=$(get_sec)
    local app_test_duration=$(get_duration ${app_test_start_time} ${app_test_end_time})

    ###2.根据小报告的合计内容，写大报告
    if [ ${APP_SETUP_SUCCESS} -eq 0 ] && [ ${FAIL_NUM} -eq 0 ]; then
        CASEPOOL_PASS_NUM=$(( ${CASEPOOL_PASS_NUM}+1 ))
        local casepool_result="通过"
    else
        CASEPOOL_FAIL_NUM=$(( ${CASEPOOL_FAIL_NUM}+1 ))
        local casepool_result="不通过"
    fi

    ### 总体用例数量统计
#    TOTAL_PASS_NUM=$(( ${TOTAL_PASS_NUM} + ${PASS_NUM} ))
#    TOTAL_FAIL_NUM=$(( ${TOTAL_FAIL_NUM} + ${FAIL_NUM} ))

    create_report_with_csv "casepool" "${REPORT_CSV_FILE}" "${casepool_para}" "${casepool_result}" "${case_num}" "${PASS_NUM}" "${FAIL_NUM}" "${SKIP_NUM}" "$(sec_to_time ${app_test_start_time})" "$(sec_to_time ${app_test_end_time})" "${app_test_duration}" "${results}"
    PASS_NUM=0
    FAIL_NUM=0
    WARN_NUM=0
    SKIP_NUM=0
}

#方法用途：循环执行测试用例，将测试结果写入report.csv
#入参：param1 数组类型
function run_case(){
    local file_paths=($1)
    local casepool_exec_times=0
    local casepool_null=0

    # 获取用例列表长度
    local case_list_length=${#file_paths[@]}
    if [ ${case_list_length} -eq 0 ]; then #如果待执行的测试用例个数为0，则退出测试并返回1
        log_info "[Blazers]没找到运行的用例脚本，退出测试" | tee -a ${RUN_LOG_FILE}
        exit 1
    fi

    log_info "[Blazers]正在执行测试用例..." | tee -a ${RUN_LOG_FILE}

    # 生成报告标题
    if [ ${CONTINUE} -eq 0 ]; then
        local title='"测试集","测试结果","合计","成功","失败","跳过","开始时间","结束时间","测试时长"'
        for file in ${file_paths[@]}
        do
            CASE_NAME=$(get_value_from_case_info_by_key ${file} "case_name")
            title=${title}',"'${CASE_NAME}'"'
#            CASE_NUM=$(get_case_num_from_case_info ${path})
#            title=${title}',"'${CASE_NUM}'"'
        done
        create_report_with_csv "title" ${REPORT_CSV_FILE} ${title}
    fi

    ###拿casepool的循环
    ###1.获取appname，没有appname时，使用“第X次”
    ###2.根据步骤1的内容，定义小报告的名称(公共方法)与目录/report/subreport
    ###3.获取case

    # 首先在control文件中查找casepool执行的配置
    casepool_exec_times=$(get_para_from_control "casepool")
    if [ "${casepool_exec_times}" = "" ]; then
        # 如果没有配置，默认执行1次
        casepool_exec_times=1
    fi

    # 如果指定casepool执行次数
    if [[ $(echo "${casepool_exec_times}" | grep -E '^[1-9][0-9]*$') ]]; then
        log_info "[Blazers]从控制器读取 casepool 执行次数: ${casepool_exec_times}" | tee -a ${RUN_LOG_FILE}
        change_setting "CONTINUE" 1

        # 设置用例执行总数
        set_total_case_num "$[ ${case_list_length} * ${casepool_exec_times} ]"

        # 判断是否分批
        if [ ${BATCH} -eq 0 ]; then
            for((j=${EXEC_INDEX};j<=${casepool_exec_times};j++))
            do
                CASE_REPORT="$(pwd)/report/subreport/${j}_report.csv"
                rm -f ${CASE_REPORT}
                create_report_with_csv "title" ${CASE_REPORT} '"测试类型","用例编号","用例名称","用例描述","测试结果","测试用时","发现错误","未修复时会产生影响","如何修复"'
                exec_casepool "${file_paths[*]}" "$j"
                change_setting "EXEC_INDEX" "$[ ${EXEC_INDEX} + 1 ]"
            done
        else # 分批
            for((j=${EXEC_INDEX};j<=${casepool_exec_times};j++))
            do
                CASE_REPORT="$(pwd)/report/subreport/${j}_report.csv"
                rm -f ${CASE_REPORT}
                create_report_with_csv "title" ${CASE_REPORT} '"测试类型","用例编号","用例名称","用例描述","测试结果","测试用时","发现错误","未修复时会产生影响","如何修复"'
                exec_casepool "${file_paths[*]}" "$j"
                change_setting "EXEC_INDEX" "$[ ${EXEC_INDEX} + 1 ]"
                change_setting "BATCH_EXEC_NUMBER" "$[ ${BATCH_EXEC_NUMBER} + 1 ]"

                if [ ${BATCH_EXEC_NUMBER} -eq ${BATCH_NUMBER} ]; then
                    change_setting "BATCH_EXEC_NUMBER" 0
                    # 重启
                    local tool_path=$(pwd)
                    local des_path=${tool_path%/*}
                    cd ${des_path}/sysrestore
                    echo ${PASSWD} | sudo -S ./run_recover.sh
                    cd - > /dev/null
                fi
            done

            # 测试完毕计数归零
            if [ ${BATCH_EXEC_NUMBER} -ne 0 ]; then
                change_setting "BATCH_EXEC_NUMBER" 0
            fi
        fi
    # 如果指定casepool按文件执行
    elif [ -f ${casepool_exec_times} ]; then
        if [ ${CONTINUE} -eq 0 ]; then
            # 待测列表长度
            list_length=$(cat ${casepool_exec_times} | wc -l)
            log_info "[Blazers]从控制器读取 casepool 执行文件: ${casepool_exec_times} 执行文件行数: ${list_length}" | tee -a ${RUN_LOG_FILE}
            change_setting "LIST_LENGTH" "${list_length}"
            change_setting "CONTINUE" 1

            # 设置用例执行总数
            set_total_case_num "$[ ${case_list_length} * ${list_length} ]"
        fi
        # 判断是否分批
        if [ ${BATCH} -eq 0 ]; then
            while [ ${EXEC_INDEX} -le ${LIST_LENGTH} ]
            do
                local app_path=$(cat ${casepool_exec_times} | sed -n ''"${EXEC_INDEX}"'p')
                local app_name=$(get_subreport_name "${app_path}")
                CASE_REPORT="$(pwd)/report/subreport/${app_name}_report.csv"
                rm -f ${CASE_REPORT}
                create_report_with_csv "title" ${CASE_REPORT} '"测试类型","用例编号","用例名称","用例描述","测试结果","测试用时","发现错误","未修复时会产生影响","如何修复"'
                exec_casepool "${file_paths[*]}" "${app_path}"
                change_setting "EXEC_INDEX" "$[ ${EXEC_INDEX} + 1 ]"
            done
        else
            while true #[ ${EXEC_INDEX} -le ${LIST_LENGTH} ]
            do
                local app_path=$(cat ${casepool_exec_times} | sed -n ''"${EXEC_INDEX}"'p')
                local app_name=$(get_subreport_name "${app_path}")
                CASE_REPORT="$(pwd)/report/subreport/${app_name}_report.csv"
                rm -f ${CASE_REPORT}
                create_report_with_csv "title" ${CASE_REPORT} '"测试类型","用例编号","用例名称","用例描述","测试结果","测试用时","发现错误","未修复时会产生影响","如何修复"'
                exec_casepool "${file_paths[*]}" "${app_path}"
                change_setting "EXEC_INDEX" "$[ ${EXEC_INDEX} + 1 ]"
                change_setting "BATCH_EXEC_NUMBER" "$[ ${BATCH_EXEC_NUMBER} + 1 ]"

		        if [ ${EXEC_INDEX} -gt ${LIST_LENGTH} ]; then
		            break
		        fi

                if [ ${BATCH_EXEC_NUMBER} -eq ${BATCH_NUMBER} ]; then
                    change_setting "BATCH_EXEC_NUMBER" 0
                    # 重启
                    local tool_path=$(pwd)
                    local des_path=${tool_path%/*}
                    cd ${des_path}/sysrestore
                    #read -p "完成后按任意键继续！"
                    echo ${PASSWD} | sudo -S ./run_recover.sh
                    cd - > /dev/null
                fi
            done

            # 测试完毕计数归零
            if [ ${BATCH_EXEC_NUMBER} -ne 0 ]; then
                change_setting "BATCH_EXEC_NUMBER" 0
            fi
        fi
    # 其他情况为写了配置文件但不符合以上情况的，理解为配置文件填写错误，重新填写后再执行
    else
        log_error "[Blazers]casepool 配置运行选项不符合规范，请修改后重新运行程序" | tee -a ${RUN_LOG_FILE}
        exit 1
    fi
    echo "########## 测试环境信息报告路径为 ${ENVIRONMENT_CSV_FILE} ##########"
    echo "########## 项目测试报告路径为 ${REPORT_CSV_FILE} ##########"

    ###4.大报告的合计
    local subreports=($(ls report/subreport | grep -v .gitkeep))
    local total_pass_num=0
    local total_fail_num=0
    local total_skip_num=0
    for subreport in ${subreports[*]}
    do
        local results=($(cat report/subreport/${subreport} | grep '\".*,.*\"' | awk -F ',' '{print $5}' | sed '1d'))
        for result in ${results[*]}
        do
            case ${result} in
            "\"通过\""|"\"警告\"")
                total_pass_num=$[ ${total_pass_num} + 1 ]
            ;;
            "\"不通过\""|"\"未知错误\"")
                total_fail_num=$[ ${total_fail_num} + 1 ]
            ;;
            "\"不涉及\"")
                total_skip_num=$[ ${total_skip_num} + 1 ]
            ;;
            esac
        done
    done
    log_info "[Blazers]Total Test Case :${EXEC_CASE_NUM} 条============PASS:${total_pass_num}=====================FAIL:${total_fail_num}=====================NA:${total_skip_num}============" | tee -a ${RUN_LOG_FILE}
    log_info "[Blazers]测试用例执行完毕" | tee -a ${RUN_LOG_FILE}
}



#调试--调用实例：
# function get_suit_name_from_path
#path="./TestDemo/TestSuit2/testcase05.sh "
#myname=$(get_suit_name_from_path $path)
#echo "my suit is:"$myname

#caselist=$(get_case_list_contain_aANDb smoke)
#echo "All the test case :" $caselist

#caselist=$(get_case_list_contain_aANDb smoke)
#echo "All the test case :" $caselist

#调用实例：检索出只含有smoke的测试用例
#factor="smoke"
#res=""
#res=($(get_case_include $factor))
#arg1=$(echo ${res[*]})
#echo "arg1:"${arg1}
#echo "arg1:"${#arg1[@]}
#
#for item in ${arg1[@]}
#do
# echo "item is:"$item
#done

#echo "value is----:"${res[*]}
#echo "num is----:"${#res[@]}

#res=""
#res=($(get_case_exclude $factor))
#echo "value is----:"${res[*]}
#echo "num is----:"${#res[@]}
#arg1=($(echo ${res[@]}))
#echo $arg1

#result=$(run_case $arg1)
#echo $result

#执行测试用例
#factor="smoke"
#res=""
#res=($(get_case_include $factor))
#result=$(run_case "${res[*]}")
#echo $result
