#!/bin/bash
#######################################################################################
#Function: function of system backup or recover
#Platform:arm64\x86\mips
#Author: Yikun Zhang
#Date: 2021/04/12
#Company: Kylin Software Co.,Ltd.
#Description:
# 此文件共两个函数，分别为系统备份和系统恢复，可实现系统的备份和恢复功能。
########################################################################################
source ./log.sh

#系统备份
#系统备份使用方法：system_backup
#wirte by zhangyikun
function system_backup(){

	if [ `whoami` != "root" ];then
	  log ERROR "请切换到root账户运行" | tee -a $run_log
	  exit 1
	fi
        ls_date=`date +%Y%m%d`
        backup_file_source="Backup_${ls_date}"

		if [ -d "/${backup_file_source}" ];then
			log ERROR "已存在备份文件，若仍需备份，请手动删除$backup_file_source文件夹" | tee -a $run_log
			exit 1
		else
			echo "开始备份，预计5到10分钟，请勿做任何操作" |tee -a $run_log
			rsync -Pa / /$backup_file_source/ --exclude=/proc/* --exclude=/sys/* --exclude=/run/* --exclude=/dev/* --exclude=/media/* --exclude=/$backup_file_source/* > tmp.log 2>&1
			if [ $? == "0" ];then
				log INFO "备份成功" | tee -a $run_log  #上条命令执行成功
				cat tmp.log >> $run_log
				mkdir /$backup_file_source/$backup_file_source
			else
				log ERROR "	备份失败" | tee -a $run_log  #上条命令执行失败
				echo "	错误信息如下：" >> $run_log
				cat tmp.log >> $run_log
				exit 1
			fi

		fi



                }

#系统还原使用方法：system_recover /Backup20210322
#请保证备份文件路径正确
#wirte by zhangyikun
function system_recover(){

	if [ `whoami` != "root" ];then
	  log ERROR "请切换到root账户运行" | tee -a $run_log
	  exit 1
	fi
        backup_file_source=''
        backup_file_source="${1}"

		if [ ! -d "${backup_file_source}" ];then
			log ERROR "已备份文件路径不存在,请输入备份文件路径，如：/Backup20210322" | tee -a $run_log
		else
			if [[ $backup_file_source =~ /$ ]];then
				new_backup_file_source="${backup_file_source}"
			else
				new_backup_file_source="${backup_file_source}/"
			fi
			blank_folder=`echo ${new_backup_file_source}`
			rsync -Pa --delete $new_backup_file_source / --exclude=boot/* --exclude=proc/* --exclude=dev/* --exclude=run/* --exclude=sys/* --exclude=etc/fstab --exclude=media/* --exclude=${blank_folder}* > tmp.log 2>&1

			if [ $? == "0" ];then
				log INFO "还原成功"| tee -a $run_log  #上条命令执行成功
				cat tmp.log >> $run_log
				echo "10秒后自动重启系统"
				sleep 10
				echo "开始重启系统"
				#重启系统命令
				echo 1 > /proc/sys/kernel/sysrq
				echo b > /proc/sysrq-trigger
			else
				log ERROR "还原失败" | tee -a $run_log  #上条命令执行失败
				echo "错误信息如下：" >> $run_log
				cat tmp.log >> $run_log
				exit 1
			fi
		fi

}
