#!/bin/bash
#######################################################################################
#Function: function of log about log level
#Platform:amd64\arm64\x86
#Author: Fenfen Liu
#Date: 2021/3/2
#Company: Kylin Software Co.,Ltd.
########################################################################################
# 日志级别，优先级由低到高分别为：ALL<DEBUG<INFO<WARN<ERROR<CRITICAL，分别赋值，用于后续日志等级大小比较
LOG_LEVEL_ALL_NUM=1
LOG_LEVEL_DEBUG_NUM=2
LOG_LEVEL_INFO_NUM=3
LOG_LEVEL_WARN_NUM=4
LOG_LEVEL_ERROR_NUM=5
LOG_LEVEL_CRITICAL_NUM=6
# Set the standardization of log, written by Fenfen Liu
function log()
{
    local log_level=$1
    local log_info=$2
    local line=$3
    local script_name=$(basename $0)
    case ${log_level} in
      "INFO")
        echo -e "$(date '+%Y-%m-%d %T') [INFO]: ${log_info}"
        ;;
      "WARN")
        echo -e "$(date '+%Y-%m-%d %T') [WARN]: ${log_info}"
        ;;
      "ERROR")
        echo -e "$(date '+%Y-%m-%d %T') [ERROR ${script_name} ${FUNCNAME[1]}:$line]: ${log_info}"
        ;;
      *)
        echo -e "${@}"
        ;;
    esac
}

# 测试结果展示，入参（param1，测试套件及测试用例名，param2：测试结果）
function res_log() {
	local test_suite_and_case_name=$1
	local test_res=$2
	if [[ "$#" == "1" ]]; then
		echo "==========$1=========================================================="
	elif [[ "$#" == "2" ]]; then
		echo "==========$1============================================$2=========="
	fi
}

# 定义log生成格式：时间 日志等级：日志信息，例如2021-04-15 14:10:09 [DEBUG]: debug日志记录
# 入参（param1：字符串（当前日志信息等级）；param2：字符串（当前日志等级的数值）；param3：字符串（日志信息）；
# 示例：write_log "DEBUG" $LOG_LEVEL_DEBUG_NUM "$@"
function write_log()
{
	local level_name="$1"
	local level_num="$2"
	shift 2
	# 判断当前待记录的日志是否比配置文件中的日志等级高，如果高，输出日志，否则不计入日志文件
	if [ ${LOG_LEVEL_NUM} -le $level_num ]; then
  		timestamp="$(date +%Y-%m-%d\ %H:%M:%S)"
  		echo "$timestamp [$level_name]: $@"
	fi
}

# 输出DEBUG级别日志
function log_debug()
{
	 write_log "DEBUG" $LOG_LEVEL_DEBUG_NUM "$@"
}

# 输出INFO级别日志
function log_info()
{
	  write_log "INFO" $LOG_LEVEL_INFO_NUM "$@"
}

# 输出WARN级别日志
function log_warn()
{
	  write_log "WARN" $LOG_LEVEL_WARN_NUM "$@"
}

# 输出ERROR级别日志
function log_error()
{
	  write_log "ERROR" $LOG_LEVEL_ERROR_NUM "$@"
}

# 输出critical级别日志
function log_critical()
{
	  write_log "CRITICAL" $LOG_LEVEL_CRITICAL_NUM "$@"
}

