#!/bin/bash
#######################################################################################
#Function: function of report
#Platform:amd64\arm64\x86
#Author: Fenfen Liu
#Date: 2021/4/2
#Company: Kylin Software Co.,Ltd.
#Description: 报告功能相关的封装函数，如报告有邮件发送功能
########################################################################################

source ./config/configuration.conf

# 邮件发送配置，安装mutt+msmtp，并创建配置文件,written by Fenfen Liu;
# 使用示例 email_conf
function email_conf()
{
	local soft=${EMAIL_REQUIRE[@]}
	for i in $soft
	do
		ret=$(check_install $i)
		if [ $ret == "false" ]; then
			install_dep "expect"
		fi
	done

	touch ~/.muttrc
	echo "
    set sendmail="/usr/bin/msmtp"
    set use_from=yes
    set realname="test"
    set from=$EMAIL_SENDER
    set envelope_from=yes" > ~/.muttrc
	local ret1=$(check_file "~/.muttrc")
	if [  $ret1 == "true" ]; then
		log INFO "~/.muttrc已配置完成" >> ${RUN_LOG_FILE}
	else
		log ERROR "~/.muttrc配置失败，请检查" >> ${RUN_LOG_FILE}
	fi

	touch ~/.msmtprc
	touch ~/.msmtp.log
	echo "
    account default
    host smtp.kylinos.cn
    from $EMAIL_SENDER
    auth plain
    user $EMAIL_SENDER
    password $EMAIL_SEND_PASSWORD
    logfile ~/.msmtp.log" > ~/.msmtprc
	local ret2=$(check_file "~/.msmtprc")
	if [  $ret2 == "true" ]; then
		log INFO "~/.msmtprc已配置完成" >> ${RUN_LOG_FILE}
	else
		log ERROR "~/.msmtprc配置失败，请检查" >> ${RUN_LOG_FILE}
	fi
	chmod 600 ~/.msmtprc
	chmod 777 ~/.msmtp.log
}


# 邮件发送内容:测试报告report作为邮件正文,及框架及应用运行日志作为附件；
# 使用示例 email_contain
function email_contain()
{
	local title="适配部自动化测试报告_${PROJECT_NAME}_$(date "+%Y-%m-%d %T")"
	if [ -f ${CASE_LOG_FILE} ]; then
		mutt -s $title $EMAIL_RECEIVER < ${REPORT_HTML_FILE} -a ${RUN_LOG_FILE} -a ${CASE_LOG_FILE}
	else
		mutt -s $title $EMAIL_RECEIVER < ${REPORT_HTML_FILE} -a ${RUN_LOG_FILE}
	fi
}

# 发送邮件，配置文件中如果EMAIL_ENABLE=true, 发送邮件给EMAIL_RECEIVER
# 使用示例 email_send
function email_send()
{
	if [ $EMAIL_ENABLE == "true" ];then
		email_conf
		email_contain
	else
		log WARN "邮件发送配置未开启，请手动获取和查看测试结果" >> ${RUN_LOG_FILE}
	fi
}

