#!/bin/bash

#######################################################################################
#Function: function of rrport
#Platform:amd64\arm64\x86
#Author: Chunli Wang
#Date: 2021/04/01
#Company: Kylin Software Co.,Ltd.
########################################################################################

source ./config/configuration.conf
source ./pub/datastore.sh
source ./pub/baselib.sh

#获取硬件内存容量 written by zhangyikun
function get_mem(){
	local totalmem=0;
	for mem in /sys/devices/system/memory/memory*; do
		local online=$(cat ${mem}/online);
		if [ "$online" == "1" ] ; then
			totalmem=$((totalmem+$((0x$(cat /sys/devices/system/memory/block_size_bytes)))));
		fi
	done;
	echo $((totalmem/1024**2))
}

# 定义report生成格式，第一个参数为传入结果类型，第二个为报告，后续为结果信息
# 第一个参数为casepool，将测试结果写入总报告文件，调用示例：create_report_with_csv casepool ${report} test_suite result total_case_num pass_case_num fail_case_num not_run_case_num test_suite_start_time test_suite_end_time test_duration
# 第一个参数为case，将用例信息写入report.csv。调用示例：create_report_with_csv case ${report} case_suite casenum casename case_result case_test_duration
# 第一个参数为result,调用示例：create_report_with_csv ${report} result {report} result_message
# 其他类型的入参个数，暂未定义report格式，给出错误提示信息。
function create_report_with_csv() {
    if [ "$1" == "casepool" ]; then
        echo '"'$3'","'$4'","'$5'","'$6'","'$7'","'$8'","'$9'","'${10}'","'${11}'",'${12} >> $2
    elif [ "$1" == "case" ]; then
        echo -e '"'$3'","'$4'","'$5'","'$6'","'$7'","'$8'","'$9'","'${10}'",,"'${11}'"' | sed 's/^ //' >> $2
    elif [ "$1" == "result" ] || [ "$1" == "title" ] || [ "$1" == "hint" ]; then
        echo "$3" >> $2
    else
        log_info "[Blazers]传入结果类型为 $1，类型错误"
        return 1
    fi
}

# 获取应用中文名
# 参数：desktop文件路径
function get_name_cn()
{
	local path="$1"
	local name=$(cat ${path} | grep "Name")
	local name_zh=$(cat ${path} | grep "Name\[zh\_CN\]")
	local GenericName_zh=$(cat ${path} | grep "GenericName\[zh\_CN\]")
	if [  -n "${name_zh}" ]; then
		local get_name_result=$(cat ${path} | grep -w "Name\[zh\_CN\]" | awk -F '=' '{print $2}' | head -1)
	else
		if [ -n "${GenericName_zh}" ]; then
			local get_name_result=$(cat ${path} | grep -w "GenericName\[zh\_CN\]" | awk -F '=' '{print $2}' | head -1)
		else
			if [ -n "${name}" ]; then
				local get_name_result=$(cat ${path} | grep "Name\=" | awk -F '=' '{print $2}' | head -1 | tee -a name.log)
			else
				local get_name_result=$(cat ${path} | grep "GenericName\=" | awk -F '=' '{print $2}' | head -1 )
			fi
		fi
	fi

	echo ${get_name_result}
}

# 查找应用中文名
# 参数：应用名
function get_name_cn_process()
{
    local app=$1
    local name_result=""
    log_info "${app} 查找中文名称" >> ${CASE_LOG_FILE}
    apt-get download ${app} > /dev/null
    if [ $? -eq 0 ]; then
        log_info  "下载应用包${app}成功" >> "${CASE_LOG_FILE}"
        local pkg=$(ls | grep deb$)
        dpkg -x ${pkg} tmp
        log_info "===== ${app} 的启动文件 =====" >> ${CASE_LOG_FILE}
        local desk=$(find tmp -name *.desktop)
        if [ -z "$desk" ]; then
            log_error "此应用无desktop文件，无法记录软件中文名称" >> ${CASE_LOG_FILE}
        else
            find tmp -name *.desktop | grep -v uninstall | grep -v template > desktop.tmp
            log_info "查找到启动文件列表 $(cat desktop.tmp)" >> ${CASE_LOG_FILE}
            # 查询开始菜单下的启动路径
            log_info "优先查找开始菜单栏的desktop文件" >> ${CASE_LOG_FILE}
            cat desktop.tmp | grep "^tmp/usr/share/applications/" > menu_desktop.tmp
            if [ -n "$(cat menu_desktop.tmp)" ]; then
                desktop_path=$(cat menu_desktop.tmp | head -1)
                local name_result=$(get_name_cn ${desktop_path})
            else
                log_info "开始菜单栏无desktop文件，开始在/opt/下寻找含有中文名称的文件" >> ${CASE_LOG_FILE}
                cat desktop.tmp | grep "^tmp/opt/" > opt_desktop.tmp
                if [ -n "$(cat opt_desktop.tmp)" ]; then
                    desktop_path=$(cat opt_desktop.tmp |head -1)
                    local name_result=$($get_name_cn ${desktop_path})
                else
                    echo "可选安装目录下无desktop文件,开始在/etc/xdg/autostart寻找含有中文名称的文件" >> ${CASE_LOG_FILE}
                    cat desktop.tmp | grep "^tmp/etc/xdg/autostart" > autostart_desktop.tmp
                    if [ -n "$(cat autostart_desktop.tmp)" ]; then
                        desktop_path=$(cat autostart_desktop.tmp |head -1)
                        local name_result=$(get_name_cn ${desktop_path})
                    else
                        log_warn "常用路径下无desktop文件,无法找出软件中文名称" >> ${CASE_LOG_FILE}
                    fi
                fi
            fi

            rm -f desktop.tmp
            rm -f menu_desktop.tmp
            rm -f opt_desktop.tmp
            rm -f autostart_desktop.tmp
            rm -f exec.tmp
            rm -f after.log
            rm -f before.log
        fi

        rm -f *.deb
        rm -rf tmp

        if [ -n "${name_result}" ];then
            log_info "${app} 应用中文名称获取成功" >> ${CASE_LOG_FILE}
        else
            log_error "${app} 应用中文名称获取失败" >> ${CASE_LOG_FILE}
        fi
    else
        log_info  "下载应用包${app}失败" >> "${CASE_LOG_FILE}"
        rm -f *.FAILED
    fi
    echo ${name_result}
}

# 功能：从包路径中取出包名
# 参数：包路径
# 返回值：包名
function get_subreport_name() {
    local pkg_path="$1"
    local app_name=${pkg_path##*/}
    if [[ "${app_name}" =~ .(deb|rpm)$ ]]; then
        echo "${app_name%.*}"
    else
        echo "${app_name}"
    fi
}

# 生成html报告
# 入参：无
# 返回值：无
function generate_report() {
    log_info "[Blazers]正在生成报告..." | tee -a ${RUN_LOG_FILE}
    generate_report_from_template
    write_title_into_report
    collect_environment_info
    write_environment_info_into_report
    write_runtime_info_into_report
    write_log_info_into_report
    write_test_result_into_report
    echo "########## html报告路径为 ${REPORT_HTML_FILE} ##########"
}

# 从模板复制生成报告
# 入参:无
# 返回值：无
function generate_report_from_template() {
    cp ${REPORT_TEMPLATE} ${REPORT_HTML_FILE}
}

# 将报告标题写入报告
# 入参：报告
# 返回值：无
function write_title_into_report() {
    local report_html_file_name="${REPORT_HTML_FILE%%.*}"
    local report_title="${report_html_file_name##*/}"

    # 将数据写入报告
    sed -i ${REPORT_HTML_FILE} -e 's/report_name = \"\"/report_name = \"'${report_title}'\"/'
}

# 收集环境信息
# 入参：无
# 返回值：无
function collect_environment_info() {
#    local ip=$(ifconfig | grep -w 'inet' | grep -v '127.0.0.1' | awk '{print $2}')
#    echo -e '"IP","'${ip}'"' > ${ENVIRONMENT_CSV_FILE}
    if [ "${SYSTEM}" == "desktop" ];then
	    local sys=$(cat /etc/os-release | grep VERSION_US | awk -F '=' '{print $2}' | sed -e 's/\"//g')
	    local ver=$(cat /etc/kylin-build | head -n 2 | tail -n 1 | awk '{print $NF}')
	    local image=$(cat /etc/.kyinfo | grep dist_id | awk -F '=' '{print $2}')
    elif [ "${SYSTEM}" == "server" ];then
        local sys=$(cat /etc/os-release | grep PRETTY_NAME | awk -F '=' '{print $2}' | sed -e 's/\"//g')
        local ver=$(cat /etc/.productinfo | head -n 2 | tail -n 1 | awk -F '/' '{print $NF}')
        local image=$(nkvers | grep -A 2 Build: | sed '1d')
    fi
    echo -e '"操作系统版本","'${sys}' '${ver}'"' >> ${ENVIRONMENT_CSV_FILE}
    local arch=$(lscpu | egrep '架构|Architecture' | awk '{print $2}')
    echo -e '"操作系统架构","'${arch}'"' >> ${ENVIRONMENT_CSV_FILE}
    local core=$(uname -r)
    echo -e '"系统内核版本","'${core}'"' >> ${ENVIRONMENT_CSV_FILE}
    local cpu=$(cat /proc/cpuinfo | grep 'model name' | uniq | awk -F ':' '{print $2}' | xargs)
    echo -e '"CPU型号","'${cpu}'"' >> ${ENVIRONMENT_CSV_FILE}
    local cpu_core=$(grep -c 'model name' /proc/cpuinfo)
    echo -e '"CPU核数","'${cpu_core}'"' >> ${ENVIRONMENT_CSV_FILE}
    local memory=$(get_mem)
    echo -e '"内存容量","'${memory}' MB"' >> ${ENVIRONMENT_CSV_FILE}
    local disk_size=$(echo ${PASSWD} | sudo -S fdisk -l | grep 'Disk /dev/' | awk '{print $2}')
    echo -e '"硬盘容量","'${disk_size}' GB"' >> ${ENVIRONMENT_CSV_FILE}
    local resolution=$(xrandr | head -1 | cut -d ',' -f2 | awk '{printf $2 $3 $4}')
    echo -e '"系统分辨率","'${resolution}'"' >> ${ENVIRONMENT_CSV_FILE}
    local screen=$(xrandr | sed -n '2p' | awk '{print $1}')
    echo -e '"当前显示器","'${screen}'"' >> ${ENVIRONMENT_CSV_FILE}
    local vga=$(lspci | grep -i VGA)
    echo -e '"显卡型号","'${vga#*}'"' >> ${ENVIRONMENT_CSV_FILE}
    echo -e '"镜像名称","'${image}'"' >> ${ENVIRONMENT_CSV_FILE}
}

# 将环境信息写入报告
# 入参：报告
# 返回值：无
function write_environment_info_into_report() {
    local enviroment_info=""
    while read line
    do
        local tmp_env_info=${line/","/":"}
        if [ "${environment_info}"x = ""x ]; then
            environment_info=${tmp_env_info}
        else
            environment_info=${environment_info},${tmp_env_info}
        fi
    done < ${ENVIRONMENT_CSV_FILE}
    environment_info="{"${environment_info}"}"

    # 特殊字符转义处理
    environment_info=${environment_info//\"/\\\"}
    environment_info=${environment_info//\//\\\/}

    # 将数据写入报告
    sed -i ${REPORT_HTML_FILE} -e "s/env_dic = \"\"/env_dic = ${environment_info}/"
}

# 将运行信息写入报告
# 入参：报告
# 返回值：无
function write_runtime_info_into_report() {
    local start_time=$(sec_to_time ${START_TIME})
    local end_time=$(sec_to_time ${END_TIME})
    local time_duration=$(get_duration ${START_TIME} ${END_TIME})
    local runtime_info="{\"测试开始时间\":\"${start_time}\",\"测试结束时间\":\"${end_time}\",\"测试时长\":\"${time_duration}\"}"

    # 将数据写入报告
    sed -i ${REPORT_HTML_FILE} -e "s/run_dic = \"\"/run_dic = ${runtime_info}/"
}

# 将日志信息写入报告
# 入参：报告
# 返回值：无
function write_log_info_into_report() {
    local run_log_name=$(echo ${RUN_LOG_FILE} | awk -F "/" '{print $NF}')
    local case_log_path=${CASE_LOG_FILE%/*}
    local case_log_name=$(echo ${case_log_path} | awk -F "/" '{print $NF}')

    local log_info="{\"框架日志文件\":{\"日志名\":\"${run_log_name}\",\"日志路径\":\"${RUN_LOG_FILE}\"},\"用例日志文件\":{\"日志名\":\"${case_log_name}\",\"日志路径\":\"${case_log_path}\"}}"

    # 绝对路径替换成相对路径
    log_info=${log_info//$(pwd)/..}

    # 特殊字符转义处理
    log_info=${log_info//\//\\\/}

    # 将数据写入报告
    sed -i ${REPORT_HTML_FILE} -e "s/log_dic = \"\"/log_dic = ${log_info}/"
}

# 将测试结果写入报告
# 入参：报告
# 返回值：无
function write_test_result_into_report() {
    local all_test_result=""
    while read line
    do
        if [ "${line%%,*}"x != '"测试集"'x ] && [ "${line}"x != ""x ]; then
            local test_suite=$(echo ${line} | awk -F ',' '{print $1}')
            local test_result=$(echo ${line} | awk -F ',' '{print $2}')
            local test_case_num=$(echo ${line} | awk -F ',' '{print $3}' | sed 's/"//g')
            local test_pass_num=$(echo ${line} | awk -F ',' '{print $4}' | sed 's/"//g')
            local test_fail_num=$(echo ${line} | awk -F ',' '{print $5}' | sed 's/"//g')
            local test_na_num=$(echo ${line} | awk -F ',' '{print $6}' | sed 's/"//g')
            local test_start_time=$(echo ${line} | awk -F ',' '{print $7}')
            local test_time=$(echo ${line} | awk -F ',' '{print $9}')
            local test_result_detail=$(get_test_result_detail_info ${test_suite//\"/})

            local single_test_result='{"测试集":'${test_suite}',"合计":'${test_case_num}',"成功":'${test_pass_num}',"失败":'${test_fail_num}',"跳过":'${test_na_num}',"测试结果":'${test_result}','
            single_test_result=${single_test_result}'"开始时间":'${test_start_time}',"测试时长":'${test_time}',"用例集":'${test_result_detail}'}'

            # 特殊字符转义处理
            # 引号
            single_test_result=${single_test_result//\"/\\\"}
            # 斜杠
            single_test_result=${single_test_result//\//\\\/}

            sed -i ${REPORT_HTML_FILE} -e "/result_json = /s/]$/${single_test_result},]/"
        fi
    done < ${REPORT_CSV_FILE}

    # 去除多余的逗号
    sed -i ${REPORT_HTML_FILE} -e "/result_json = /s/,]$/]/"
}

# 输出测试集结果信息
# 入参:测试集名称
# 返回值：测试集测试详细结果
function get_test_result_detail_info() {
    local test_subreport_file=$(get_subreport_name $1)_report.csv
    local report_file="./report/subreport/${test_subreport_file}"
    local subreport_info=""

    ### 获取本条用例开始行和下一条用例开始行之间的数据，即本条用例的数据
    # 获取case_num列，用作标识符，筛选行条件为"xxx,xxx"
    local case_nums=($(cat ${report_file} | grep '\".*,.*\"' | awk -F ',' '{print $2}' | sed '1d'))
    # 遍历数组
    local length=${#case_nums[*]}
    for ((i=1; i<=${length}; i++))
    do
        # 本条用例标识符
        local index1=$[i-1]
        # 找到标识符所在行，数据截取的开头
        local line1=($(cat ${report_file} | grep -n ${case_nums[index1]} | awk -F ':' '{print $1}'))
        # 找到下一条用例的开始行，作为截止行
        if [ ${i} -eq ${length} ]; then
            line2=$(cat ${report_file} | wc -l)
        else
            index2=${i}
            line2=($(cat ${report_file} | grep -n ${case_nums[index2]} | awk -F ':' '{print $1}'))
            line2=$[line2-1]
        fi
        # 截取数据行
        local log=$(cat ${report_file} | sed -n ''${line1}','${line2}'p' | sed 's/\r/\\n/g' | sed 's/$/\\n/g' | sed 's/\\n\\n/\\n/g' | sed 's/\"\\n/\"/g')
        # 获取各列数据，分割符为","
        local case_num=$(echo ${log} | awk -F '\",\"' '{print $2}')
        local case_name=$(echo ${log} | awk -F '\",\"' '{print $3}')
        local case_result=$(echo ${log} | awk -F '\",\"' '{print $5}')
        local case_error_log=$(echo ${log} | awk -F '\",\"' '{print $7}' | sed 's/\\n /\\n/g' | sed 's/\\n/\\\\n/g' | sed 's/\"$//' | sed 's/\"\"/\"/g' | sed 's/\"/\\\"/g')
        if [ ${#case_error_log} -gt ${ERROR_LOG_SIZE} ]; then
            case_error_log="错误日志过长，不能完整显示，请您到"${report_file}"中查看"
        fi

        local single_case_result_info='{"用例编号":"'${case_num}'","用例名称":"'${case_name}'","测试结果":"'${case_result}'","错误日志":"'${case_error_log}'"}'

        if [ "${subreport_info}"x == ""x ]; then
            subreport_info=${single_case_result_info}
        else
            subreport_info=${subreport_info},${single_case_result_info}
        fi
    done

    echo '['${subreport_info}']'
}