#!/bin/bash
#######################################################################################
# 框架测试入口函数
#Platform:amd64\arm64\x86
#Author: Fenfen Liu
#Date: 2021/3/2
#Company: Kylin Software Co.,Ltd.
########################################################################################
source ./pub/baselib.sh
source ./config/configuration.conf
source ./config/requirements.txt
source ./pub/report.sh
source ./pub/log.sh
source ./pub/newsnotify.sh
source ./pub/caseschedule.sh

function setup()
{
    # 配置configuration.ini中带时间戳的log和report文件路径
    get_start_sec
    write_config_file $START_TIME

    # 用例执行次数清零
    set_zero_case_num

    # 判断系统平台
    get_system

    # 检查环境配置
    # check_kylinsec

    # 清除log和report
    local basepath=$(pwd)
    local log_path=${basepath}/log
    local case_log_path=${log_path}/caselog
    local report_path=${basepath}/report
    local subreport_path=${report_path}/subreport
    remove_expired_files ${log_path} 1 # 保留已生成的本次日志
    remove_expired_files ${case_log_path} 0
    remove_expired_files ${report_path} 0
    remove_expired_files ${subreport_path} 0
}

# 执行项目初始化脚本
# 入参：项目初始化脚本参数
function project_setup(){
    local project_setup_option=( $* )
    local project_setup_path=./${PROJECT_NAME}/setup/project_setup.sh

    log_info "[Blazers]搜索项目初始化脚本" | tee -a ${RUN_LOG_FILE}
    if [ -f ${project_setup_path} ]; then
        log_info "[Blazers]执行项目初始化脚本" | tee -a ${RUN_LOG_FILE}
#        bash ${project_setup_path} "${project_setup_option[@]}"
        ${project_setup_path} "${project_setup_option[@]}"
        if [ $? -eq 0 ]; then
            log_info "[Blazers]项目初始化成功" | tee -a ${RUN_LOG_FILE}
        else
            log_info "[Blazers]项目初始化失败" | tee -a ${RUN_LOG_FILE}
            log_info "[Blazers]退出执行" | tee -a ${RUN_LOG_FILE}
            return 1
        fi
    else
        log_info "[Blazers]未找到项目初始化脚本" | tee -a ${RUN_LOG_FILE}
    fi
    return 0
}

function teardown()
{
    # 往次测试结果的定期删除（配置文件中可配置保留文件个数）
    log_info "[Blazers]正在清除过期文件..." | tee -a ${RUN_LOG_FILE}
    basepath=$(pwd)
    local pack_dir=$basepath/pack
    remove_expired_files $pack_dir ${FILE_RESERVE_NUM}
    log_info "[Blazers]文件清理完毕" | tee -a ${RUN_LOG_FILE}

    if [ ${BATCH} -eq 1 ]; then
        remove_autostart
    fi

    change_setting "EXEC_INDEX" 0
    change_setting "CONTINUE" 0
}

# 主函数
function main() {
    #读取option函数，新设变量：number
    local case_options=""
    local config_options=""
    local other_options=""

    while [ -n "$1" ]; do
        case $1 in
            --run|-run)
                shift
                if [ -z "${case_options}" ]; then
                    case_options="--run $1"
                fi
                ;;
            --include|-include)
                shift
                if [ -z "${case_options}" ]; then
                    case_options="--include $1"
                fi
                ;;
            --exclude|-exclude)
                shift
                if [ -z "${case_options}" ]; then
                    case_options="--exclude $1"
                fi
                ;;
            -p|--passwd|-passwd)
                shift
                if [ -z "${config_options}" ]; then
                    config_options="-p $1"
                fi
                ;;
            -h|--help)
                shift
                help
                exit
                ;;
            *)
                other_options=${other_options}" $1"
                ;;
        esac
        shift
    done

    # 场景1：第一次使用，传入必需参数，从头执行
    # 场景2：执行过程中断，直接运行run.sh，继续执行
    # 场景3：完整执行后，重复之前的测试，参数不变，可以直接运行run.sh（暂不支持）

    if [ "${config_options}"x == ""x ] && [ "${other_options}"x == ""x ] && [ "${case_options}"x == ""x ]; then
        # 继续跑还是从头开始由continue控制
        if [ "${LIST_LENGTH}"x == ""x ] || [ "${PASSWD}"x == ""x ]; then
            log_error "[Blazers]看起来您好像遗忘了配置参数，请检查后再次使用" | tee -a ${RUN_LOG_FILE}
            exit
        fi
    else
        change_setting "EXEC_INDEX" 0
        change_setting "CONTINUE" 0
    fi

    # 判断网络状态
    check_network ${CHECK_NETWORK_IP}
    ret=$?

    if [ ${CONTINUE} -eq 0 -a ${ret} -eq 0 ]; then
        set_password ${config_options}
        setup
        project_setup ${other_options}

        if [ ${BATCH} -eq 1 ]; then
            change_setting "EXEC_NUMBER" 0
            set_autologin
            set_autostart
            backup
        fi
        ret=$?
    fi

    # 判断项目初始化是否成功
    if [ ${ret} -eq 0 ]; then
        log_info "[Blazers]正在获取用例列表..." | tee -a ${RUN_LOG_FILE}
        if [ ${CONTINUE} -eq 0 ]; then
            change_setting "CASE_OPTIONS" "\"${case_options}\""
            change_setting "EXEC_INDEX" "$[ ${EXEC_INDEX} + 1 ]"
        fi
        local case_list=($(get_case ${CASE_OPTIONS}))
        local cases=($(get_case_list_contain_system ${case_list[*]}))
        log_info "[Blazers]获取用例列表完成" | tee -a ${RUN_LOG_FILE}
        run_case "${cases[*]}"
    fi

    # 全部测试执行完毕后执行
    if [ ${EXEC_INDEX} -gt ${LIST_LENGTH} ]; then
        # report.csv转html
        get_end_sec
        generate_report
        package_result ${START_TIME}
        # 邮件发送
        email_send
        teardown
    fi
}

# 主函数部分
args=( $* )
# shellcheck disable=SC2068
main ${args[@]}