#!/bin/bash
################################################################################
#Function: recover system
#Platform:amd64/arm64/x86
#Author: Zhang Yikun
#Date: 2021/11/09
#Company: Kylin Software Co.,Ltd.
################################################################################
run_log=log.log
#EXCLUDEFILE='exclude_auto'
RESTORE_EXCLUDEFILE='exclude_user_auto'
# Set the standardization of log, written by Fenfen Liu
function log()
{
    local log_level=$1
    local log_info=$2
    local line=$3
    local script_name=$(basename $0)
    case ${log_level} in
      "INFO")
        echo -e "$(date '+%Y-%m-%d %T') [INFO]: ${log_info}"
        ;;
      "WARN")
        echo -e "$(date '+%Y-%m-%d %T') [WARN]: ${log_info}"
        ;;
      "ERROR")
        echo -e "$(date '+%Y-%m-%d %T') [ERROR ${script_name} ${FUNCNAME[1]}:$line]: ${log_info}"
        ;;
      *)
        echo -e "${@}"
        ;;
    esac
}

#请先运行system_backup将系统备份。
#example:system_recover /backup0329/
function system_recover(){

	if [ `whoami` != "root" ];then
	  log ERROR "请切换到root账户运行" | tee -a $run_log
	  exit 1
	fi
        backup_file_source=''
        backup_file_source="${1}"
	
		if [ ! -d "${backup_file_source}" ];then
			log ERROR "[autotest] 已备份文件路径不存在,请输入备份文件路径，如：/Backup20210322" | tee -a $run_log
		else
			if [[ $backup_file_source =~ /$ ]];then
				new_backup_file_source="${backup_file_source}"
			else
				new_backup_file_source="${backup_file_source}/"
			fi
			#blank_folder=`echo ${new_backup_file_source#*/}`
			log INFO "[autotest] 开始还原"| tee -a $run_log
			rsync -avAXHr --no-inc-recursive --ignore-missing-args --delete --exclude-from ${RESTORE_EXCLUDEFILE} ${new_backup_file_source} / 1> tmp.log 2>>$run_log
			if [ $? == "0" ];then
				log INFO "[autotest] 还原成功"| tee -a $run_log  #上条命令执行成功
				echo "10秒后自动重启系统" | tee -a $run_log
				sleep 10
				echo "开始重启系统" | tee -a $run_log
				#重启系统命令
				echo 1 > /proc/sys/kernel/sysrq
				echo b > /proc/sysrq-trigger
			else
				log ERROR "[autotest] 还原失败" | tee -a $run_log  #上条命令执行失败
				log ERROR "[autotest] 错误信息请参考log.log和tmp.log"|tee -a $run_log
				exit 1
			fi
		fi

}

system_recover $1
