# coding=utf-8
"""
@Project : kylin-adaptation-tools
@Time    : 2023/8/16 15:12
@Author  : wangbin
"""
import os
import subprocess
import re
import configparser
from src.setting import SCRIPT_PATH
from src.setting import REPORT_PATH
from src.utils.utils import escape_character
from src.utils.utils import read_blazer_config
import pprint


class Operation:
    """
    操作类，实现脚本的调用以及获取返回值
    """
    # def __int__(self):
    #     self.script_path = os.path.join(SCRIPT_PATH, 'blazers')

    def step1(self):
        pass

    def step2(self):
        pass

    def _chmod(self, script_path):
        """
        给脚本入口文件赋权
        :param script_path:
        :return:
        """
        cmd = f'cd {script_path}/;chmod +x run.sh'
        fetch = subprocess.Popen(cmd, stdout=subprocess.PIPE, shell=True)

    def run_debcheck(self, package_path, package_type):
        """
        执行包规范检查工具，获取报告
        :param package_path
        :param package_type
        :return report_file
        """

        script_path = os.path.join(SCRIPT_PATH, 'kylindebcheck')
        self._chmod(script_path)

        run_tool_cmd = f'cd {script_path}/; ./run.sh {package_type} {package_path}'
        fetch = subprocess.Popen(run_tool_cmd, stdout=subprocess.PIPE, shell=True)
        fetch = fetch.stdout.read().decode('utf-8')
        report_detail = re.search('测试详细结果路径为.*\.csv', fetch).group().replace('测试详细结果路径为: ', '')
        report_summary = re.search('测试结果概述路径为.*\.csv', fetch).group().replace('测试结果概述路径为: ', '')
        report_pkg_info = re.search('包信息的保存路径为.*\.csv', fetch).group().replace('包信息的保存路径为: ', '')

        # print(f'包规范检查工具的结果路径为{report_detail}, {report_summary}, {report_pkg_info}')

        return report_detail, report_summary, report_pkg_info

    def run_blazers(self, package_path, passwd):
        """
        执行脚本，获取返回值
        :param package_path:
        :param passwd:
        :return:
        """
        script_path = os.path.join(SCRIPT_PATH, 'blazers')
        self._chmod(script_path)
        passwd = escape_character(passwd)
        cmd = f'cd {script_path}/;./run.sh -p {passwd} -pkg {package_path}'
        # print(cmd)
        fetch = subprocess.Popen(cmd, stdout=subprocess.PIPE, shell=True)
        fetch = fetch.stdout.read().decode('utf-8')
        # pprint.pprint(fetch)
        # report_html = re.search('html报告路径为.*\.html', fetch).group().replace('html报告路径为 ', '')
        report_env = re.search('测试环境信息报告路径为.*\.csv', fetch).group().replace('测试环境信息报告路径为 ', '')
        report_case = re.search('应用测试报告路径为.*\.csv', fetch).group().replace('应用测试报告路径为 ', '')

        return report_env, report_case

    def read_percent(self):
        """
        获取blazer工具执行百分比
        :return:
        """

        pre_step = ['run_debcheck']
        post_step = ['get_report']

        script_path = os.path.join(SCRIPT_PATH, 'blazers')
        file_path = os.path.join(script_path, 'config', 'configuration.conf')
        total = read_blazer_config(file_path, 'TOTAL_CASE_NUM')
        current = read_blazer_config(file_path, 'EXEC_CASE_NUM') or 0
        # print(f'测试用例总数是{total};', f'当前已执行用例{current};', '+' * 100)
        percent = (int(current) + len(pre_step)) / (int(total) + len(pre_step) + len(post_step)) * 100
        # print(f'测试用例总数是{total};', f'当前已执行用例{current};', f'百分比{percent};', '+' * 100)
        return percent

    def run_report_tool(self, env_file, std_test_report, compatibility_test_report, pkg_info_file):
        """
        :param env_file:
        :param std_test_report:
        :param compatibility_test_report:
        :param pkg_info_file:
        :param running_info_file:
        :return: report_html_file
        """

        script_path = os.path.join(SCRIPT_PATH, 'reporttool')
        # self._chmod(script_path)
        cmd = f'cd {script_path}/;python3 report.py -e {env_file} -s {std_test_report} -c {compatibility_test_report}' \
              f' -p {pkg_info_file} ' \
              # f'-r {running_info_file}'

        fetch = subprocess.Popen(cmd, stdout=subprocess.PIPE, shell=True)
        fetch = fetch.stdout.read().decode('utf-8')

        report_html = re.search('HTML报告输出至.*\.html', fetch).group().replace('HTML报告输出至：', '')
        # json_data = re.search('JSON文件输出至.*\.json', fetch).group().replace('JSON文件输出至：', '')

        return report_html